/*
 * USE OF THE LIFEx SOFTWARE IS SUBJECT TO THE FOLLOWING TERMS AND CONDITIONS. 
 * IF YOU DO NOT ACCEPT THESE TERMS AND CONDITIONS, DO NOT DOWNLOAD THE SOFTWARE. 
 * NON-PROFESSIONAL USERS ARE NOTIFIED THAT THE SOFTWARE AND THE RISKS INHERENT TO ITS USE,
 * MODIFICATION AND/OR DEVELOPMENT AND REPRODUCTION ARE NOT COVERED BY ANY WARRANTY ON THE PART OF THE CEA. 
 * IT IS HIGHLY RECOMMENDED THAT YOU ONLY LOAD AND/OR USE THE SOFTWARE IF YOU HAVE SPECIALIZED IN-DEPTH INFORMATION TECHNOLOGY KNOW-HOW.
 * THE COMMISSARIAT A L’ENERGIE ATOMIQUE ET AUX ENERGIES ALTERNATIVES (FRENCH ATOMIC ENERGY AGENCY), 
 * a public scientific, technical and industrial establishment, 
 * having its registered office at 25 rue Leblanc – bâtiment le Ponant D – 75015 Paris cedex, 
 * hereinafter referred to as "CEA", hereby licenses the right to use the LIFEx software as per the terms and conditions set out below.  
 */
package bardp.protocol.protocolTexture;

import bardp.gui.FrameGui;
import bardp.img.coor.CoorRange;
import bardp.img.controller.FrameController;
import bardp.protocol.protocolTexture.gui.TextureGlobalROIJPanel;
import bardp.protocol.protocolTexture.histo.HistogramAllSeriesGui;
import bardp.protocol.protocolTexture.io.TextureIoCsv;
import bardp.roi.controller.RoiController;
import bardp.roi.controller.RoiMain;
import bardp.session.patient.study.series.SeriesModel;
import bardp.utility.MessageGui;
import bardp.utility.WorkDir;
import java.awt.Desktop;
import java.io.File;
import java.io.IOException;
import java.util.Properties;
import java.util.logging.Level;

/**
 * <p>
 * Title: Local Image Feature Extraction (LIFEx)</p>
 * <p>
 * Description: texture processing from series and roiMain</p>
 * <p>
 * Copyright: Copyright (c) 2016</p>
 * <p>
 * Company: Inserm/CNRS/CEA</p>
 * <p>
 * @author Christophe Nioche, Inserm at lifexsoft.org
 * @version 6.0
 */
public class TextureController extends Thread {

    public TextureController(
            final SeriesModel series,
            final Properties properties,
            final String sessionImgKey
    ) {
        this.series = series;
        this.properties = properties;
        this.sessionImgKey = sessionImgKey;
    }

    @Override
    public void run() {
        //<editor-fold defaultstate="collapsed" desc="check if PicRoiManager.roiMainList.isNotEmpty">
        if (RoiController.roiMainList.isEmpty()) {
            MessageGui messageGUI = new MessageGui(
                    "Texture Protocol",
                    "Sorry, but no ROI is defined. Please define an ROI and start again",
                    MessageGui.ERROR_MESSAGE
            );
            return;
        }
        //</editor-fold>

        // define HistogramAllSeriesGui
        HistogramAllSeriesGui histogramAllSeriesGui = new HistogramAllSeriesGui();
        FrameController.setActivatedSeriesOnAllFrames(series);
        TextureGlobalROIJPanel texGlobalROIJPanel = FrameGui.protocolFrame.texBoxOption.textureJPanelOption.texGlobalROIJPanel;
        if (properties != null) {
            if (sessionImgKey != null) { // multiple series
                texGlobalROIJPanel.setPropertiesLoadedValuesFromScripting(properties, sessionImgKey);
            } else { // simple series
                texGlobalROIJPanel.setPropertiesLoadedValuesFromScripting(properties);
            }
        }

        String dimensionProcessing = texGlobalROIJPanel.dimensionProcessing;

        // processing of texture for all ROI of one series
        int nbRoiCalculated = 0;

        // copy of roiZYX for memory
        short[][] roiZYXCopy = RoiController.roiZYX.copy();

        long timeFile = System.currentTimeMillis();

         //<editor-fold defaultstate="collapsed" desc="MessageGUI: Textural processing on-going">
        MessageGui messageGui = new MessageGui(
                null,
                "Textural message",
                "processing on-going",
                MessageGui.WAIT_MESSAGE,
                100f / (series.rawTZYX.nt*9)
        );
        //</editor-fold>
        
        for (int it = 0; it < series.rawTZYX.nt; it++) {

            for (RoiMain roiMain : RoiController.roiMainList) {

                if (roiMain.roiModel.boxOptionMain.jPanelMenu.jCheckBoxEyes.isSelected()) {
                    //System.out.println("" + roiMain.roiModel.title + " " + roiMain.roiModel.description);
                    //<editor-fold defaultstate="collapsed" desc="3D processing">
                    if (dimensionProcessing.equals("3D")) {
                        // 3D management of ROI
                        // to check one quick visualisation
                        FrameController.coorCrossHair.set(roiMain.roiModel.coorMaxOnSeriesRef);
                        FrameController.allFramesUpdateWritableRasterChanged_3(null);

                        // sub processing
                        roiMain.roiModel.texControllerSub = new TextureControllerSub(
                                roiMain.roiModel,
                                series,
                                it,
                                texGlobalROIJPanel,
                                histogramAllSeriesGui,
                                messageGui
                        );
                        if (roiMain.roiModel.texControllerSub.run()) {
                            nbRoiCalculated++;
                        }
                    }
                    //</editor-fold>

                    //<editor-fold defaultstate="collapsed" desc="2D processing">
                    if (dimensionProcessing.equals("2D")) {

                        CoorRange rangeBounds = RoiController.roiZYX.getBounds(roiMain.roiModel.index);
                        for (int iz = rangeBounds.min.getZ(); iz < rangeBounds.max.getZ(); iz++) {

                            // setting again of default memory roiZYX
                            RoiController.roiZYX.arraycopy(roiZYXCopy);

                            // empty all slices except iz slice
                            for (int iiz = rangeBounds.min.getZ(); iiz <= rangeBounds.max.getZ(); iiz++) {
                                if (iiz != iz) {
                                    RoiController.roiZYX.setEmptyZ(iiz);
                                }
                            }

                            // sub processing
                            roiMain.roiModel.texControllerSub = new TextureControllerSub(
                                    roiMain.roiModel,
                                    series,
                                    it,
                                    texGlobalROIJPanel,
                                    histogramAllSeriesGui,
                                    messageGui
                            );
                            if (roiMain.roiModel.texControllerSub.run()) {
                                nbRoiCalculated++;
                            }
                            //System.out.println("iz = " + iz);
                            TextureIoCsv texIO_csv_normal = new TextureIoCsv(
                                    texGlobalROIJPanel,
                                    roiMain,
                                    it,
                                    iz,
                                    timeFile
                            );
                            texIO_csv_normal.run();  // run mandatory, no start() here
                        }
                    }
                    //</editor-fold>
                }
            }

            // setting again of default memory roiZYX
            RoiController.roiZYX.arraycopy(roiZYXCopy);

            // histogram PicRoiManager.roiMainList commun 
            if (nbRoiCalculated > 1
                    && texGlobalROIJPanel.texJFrameOutputParameters.texJPanelOutputParameters.jCheckBoxHISTO_Show.isSelected()
                    && texGlobalROIJPanel.scriptingCsv.isEmpty()) {
                histogramAllSeriesGui.setHistogramVisible();
            }

            //<editor-fold defaultstate="collapsed" desc="save result and open destination directory">
            if (nbRoiCalculated > 0) {

                if (dimensionProcessing.equals("3D")) {
                    // save results in csv format and open destination directory
                    TextureIoCsv texIO_csv_normal = new TextureIoCsv(
                            texGlobalROIJPanel,
                            null, // no roiMain for 3D dimension processing
                            it,
                            -1, // no iz for 3D dimension processing
                            timeFile
                    );
                    texIO_csv_normal.run();  // run mandatory, no start() here
                }

            } else {
                MessageGui messageGUI1 = new MessageGui(
                        "Texture Protocol",
                        "Sorry, but no ROI is visible. Please make visible an ROI and start again",
                        MessageGui.ERROR_MESSAGE
                );
            }
            //</editor-fold>

        } // end of it
        
        //<editor-fold defaultstate="collapsed" desc="open ROIValueDirectory">
        if (texGlobalROIJPanel.scriptingCsv.isEmpty()) {
            try {
                String dir = WorkDir.getROIValueDirectory(FrameController.seriesRef);
                Desktop.getDesktop().open(new File(dir));
            } catch (IOException ex) {
                FrameGui.LOGGER.log(Level.SEVERE, null, ex);
            }
        }
        //</editor-fold>
        
        messageGui.end();
        
        String sortfilename = FrameController.seriesSelected.fromStudy.fromPatient.patientName.trim().toUpperCase()
                + "_" + FrameController.seriesSelected.fromStudy.fromPatient.patientID.trim().toUpperCase()
                + "_Texture";
        sortfilename = sortfilename.replace("\\", "_");
        sortfilename = sortfilename.replaceAll("/", "_");
        MessageGui messageGUI1 = new MessageGui(
                    "csv File [ OK ]",
                    "" + sortfilename,
                    MessageGui.OK_MESSAGE
        );
    }

    private final SeriesModel series;
    private final Properties properties;
    private final String sessionImgKey;
}
