/*
 * USE OF THE LIFEx SOFTWARE IS SUBJECT TO THE FOLLOWING TERMS AND CONDITIONS. 
 * IF YOU DO NOT ACCEPT THESE TERMS AND CONDITIONS, DO NOT DOWNLOAD THE SOFTWARE. 
 * NON-PROFESSIONAL USERS ARE NOTIFIED THAT THE SOFTWARE AND THE RISKS INHERENT TO ITS USE,
 * MODIFICATION AND/OR DEVELOPMENT AND REPRODUCTION ARE NOT COVERED BY ANY WARRANTY ON THE PART OF THE CEA. 
 * IT IS HIGHLY RECOMMENDED THAT YOU ONLY LOAD AND/OR USE THE SOFTWARE IF YOU HAVE SPECIALIZED IN-DEPTH INFORMATION TECHNOLOGY KNOW-HOW.
 * THE COMMISSARIAT A L’ENERGIE ATOMIQUE ET AUX ENERGIES ALTERNATIVES (FRENCH ATOMIC ENERGY AGENCY), 
 * a public scientific, technical and industrial establishment, 
 * having its registered office at 25 rue Leblanc – bâtiment le Ponant D – 75015 Paris cedex, 
 * hereinafter referred to as "CEA", hereby licenses the right to use the LIFEx software as per the terms and conditions set out below.  
 */
package bardp.protocol.protocolTexture.calc;

import bardp.gui.FrameGui;
import bardp.img.coor.Coor;
import bardp.raw.RawTZYX;
import bardp.raw.RawSparseZYX;
import bardp.raw.RawTransformQuinticLagrangian;
import bardp.utility.Utils;
import java.util.logging.Level;
import javax.vecmath.Matrix3d;
import javax.vecmath.Vector3d;

 /**
 * <p>
 * Title: Local Image Feature Extraction (LIFEx)</p>
 * <p>
 * Description: </p>
 * <p>
 * Copyright: Copyright (c) 2016</p>
 * <p>
 * Company: Inserm/CNRS/CEA</p>
 * <p>
 * @author Christophe Nioche, Inserm at lifexsoft.org
 * @version 6.0
 */
public class CalcSpatialResamplingROI {

  public RawSparseZYX run(
          final RawSparseZYX maskOnSeriesRaw,
          final Vector3d spatialResampling
  ) {
    // result
    RawSparseZYX resMaskOnSeriesRaw;

    // message
    String msg = "\nCalcSpatialResamplingROI = Spatial ROI Resampling: ";

    if (Utils.isEqualDouble(spatialResampling.z, maskOnSeriesRaw.iopIppSp.spModel.getZSpacing())
            && Utils.isEqualDouble(spatialResampling.y, maskOnSeriesRaw.iopIppSp.spModel.getYSpacing())
            && Utils.isEqualDouble(spatialResampling.x, maskOnSeriesRaw.iopIppSp.spModel.getXSpacing())) {
      msg += "\n -> no change:";
      resMaskOnSeriesRaw = maskOnSeriesRaw.createCopy();

    } else {
      Short one = (short) 1;
      msg += "\n -> change:";
      RawTZYX rawZYXFloat = maskOnSeriesRaw.createCopyRawZYX(one);

      
      int it = 0;
      float[][] zyxFloatOutput;
      Coor nZnYnXOutput;

      Matrix3d m3d = new Matrix3d();
      m3d.setIdentity();
      FrameGui.LOGGER.log(Level.INFO, "Texture: Quintic Lagrangian Interpolation method used for series");
      RawTransformQuinticLagrangian rawZYXTransformQuinticLagrangian = new RawTransformQuinticLagrangian(
              it,
              spatialResampling, // output resolution
              rawZYXFloat,
              m3d,
              0f /* backcolor */
      );
      zyxFloatOutput = rawZYXTransformQuinticLagrangian.run();
      
      nZnYnXOutput = rawZYXTransformQuinticLagrangian.getnZYXTransformed();
      int nz = nZnYnXOutput.getZ();
      int ny = nZnYnXOutput.getY();
      int nx = nZnYnXOutput.getX();


      
      resMaskOnSeriesRaw = new RawSparseZYX(
              nz, ny, nx,
              new Vector3d(spatialResampling.x, spatialResampling.y, spatialResampling.z),
              maskOnSeriesRaw.iopIppSp.ippModel.ipp
      );

      // convert interpolated result values in binary mask of roiMain.roiModel.index
      float seuilIndex = 0.5f; // half of value 1

      int iz, iy, ix, iiy;
      for (iz = 0; iz < nz; iz++) {
        for (iy = 0; iy < ny; iy++) {
          iiy = iy * nx;
          for (ix = 0; ix < nx; ix++) {
            if (zyxFloatOutput[iz][iiy + ix] >= seuilIndex) {
              resMaskOnSeriesRaw.setZYX(iz, iy, ix, one);
            }
          }
        }
      }

      //<editor-fold defaultstate="collapsed" desc="debug: visualize result of interpolation">
      /* SeriesModel series = FrameController.seriesRef.createNewSeriesCalc("OT", "ROIInterp");
      series.rawTZYX = resMaskOnSeriesRaw.createCopyRawZYX(one);
      series.rawTZYX.updateMinMaxValues();
      series.setDefaultWindowLevelFromHistogram();
      FrameController.allColorModelChanged_1(series);
      FrameController.allSampleModelChanged_2(series);
      FrameController.allWritableRasterChanged_3(series);
      FrameController.allBufferedImageChanged_4(series);*/
      //</editor-fold>
    }

    //<editor-fold defaultstate="collapsed" desc="log">
    msg += "\n"
            + " -> spacing: " + spatialResampling.toString() + " mm, "
            + " -> dim : nz" + resMaskOnSeriesRaw.nz
            + " ny" + resMaskOnSeriesRaw.ny
            + " nx" + resMaskOnSeriesRaw.nx
            + " = tot of "
            + resMaskOnSeriesRaw.nz
            * resMaskOnSeriesRaw.ny
            * resMaskOnSeriesRaw.nx + " voxels in volume"
            + "\n";
    FrameGui.LOGGER.log(Level.INFO, msg);
    //</editor-fold>

    return resMaskOnSeriesRaw;
  }
}