/*
 * USE OF THE LIFEx SOFTWARE IS SUBJECT TO THE FOLLOWING TERMS AND CONDITIONS. 
 * IF YOU DO NOT ACCEPT THESE TERMS AND CONDITIONS, DO NOT DOWNLOAD THE SOFTWARE. 
 * NON-PROFESSIONAL USERS ARE NOTIFIED THAT THE SOFTWARE AND THE RISKS INHERENT TO ITS USE,
 * MODIFICATION AND/OR DEVELOPMENT AND REPRODUCTION ARE NOT COVERED BY ANY WARRANTY ON THE PART OF THE CEA. 
 * IT IS HIGHLY RECOMMENDED THAT YOU ONLY LOAD AND/OR USE THE SOFTWARE IF YOU HAVE SPECIALIZED IN-DEPTH INFORMATION TECHNOLOGY KNOW-HOW.
 * THE COMMISSARIAT A L’ENERGIE ATOMIQUE ET AUX ENERGIES ALTERNATIVES (FRENCH ATOMIC ENERGY AGENCY), 
 * a public scientific, technical and industrial establishment, 
 * having its registered office at 25 rue Leblanc – bâtiment le Ponant D – 75015 Paris cedex, 
 * hereinafter referred to as "CEA", hereby licenses the right to use the LIFEx software as per the terms and conditions set out below.  
 */
package bardp.protocol.protocolTexture.calc;

import bardp.gui.FrameGui;
import bardp.img.color.ColorController;
import bardp.img.coor.Coor;
import bardp.img.controller.FrameController;
import bardp.raw.RawTZYX;
import bardp.raw.RawTransformQuinticLagrangian;
import bardp.session.patient.study.series.SeriesModel;
import bardp.utility.Utils;
import java.util.logging.Level;
import javax.vecmath.Matrix3d;
import javax.vecmath.Vector3d;

/**
 * <p>
 * Title: Local Image Feature Extraction (LIFEx)</p>
 * <p>
 * Description: </p>
 * <p>
 * Copyright: Copyright (c) 2016</p>
 * <p>
 * Company: Inserm/CNRS/CEA</p>
 * <p>
 * @author Christophe Nioche, Inserm at lifexsoft.org
 * @version 6.0
 */
public class CalcSpatialResamplingSeries {

    public RawTZYX run(
            final RawTZYX rawTZYX,
            final Vector3d spatialResampling
    ) {

        String msg = "\nCalcSpatialResamplingSeries = Spatial Image Resampling: ";
        RawTZYX resRawZYX;

        int it = 0; // by construct of TexControllerSub

        rawTZYX.updateMinMaxValues(it);

        if (Utils.isEqualDouble(spatialResampling.z, rawTZYX.iopIppSp.spModel.getZSpacing())
                && Utils.isEqualDouble(spatialResampling.y, rawTZYX.iopIppSp.spModel.getYSpacing())
                && Utils.isEqualDouble(spatialResampling.x, rawTZYX.iopIppSp.spModel.getXSpacing())) {
            msg += "\n -> no change:";
            resRawZYX = rawTZYX;

        } else {
            msg += "\n -> change:";

            float[][] zyxFloatOutput;
            Coor nZnYnXOutput;

            Matrix3d m3d = new Matrix3d();
            m3d.setIdentity();

            FrameGui.LOGGER.log(Level.INFO, "Texture: Quintic Lagrangian Interpolation method used for series");
            RawTransformQuinticLagrangian rawZYXTransformQuinticLagrangian = new RawTransformQuinticLagrangian(
                    it,
                    spatialResampling, // output resolution
                    rawTZYX,
                    m3d,
                    0f /* backgroundColor */
            );
            zyxFloatOutput = rawZYXTransformQuinticLagrangian.run();
            nZnYnXOutput = rawZYXTransformQuinticLagrangian.getnZYXTransformed();

            resRawZYX = new RawTZYX(
                    1,
                    nZnYnXOutput.getZ(),
                    nZnYnXOutput.getY(),
                    nZnYnXOutput.getX(),
                    spatialResampling.z,
                    spatialResampling.y,
                    spatialResampling.x
            );

            resRawZYX.ntOnLoading = 1;
            resRawZYX.nzOnLoading = nZnYnXOutput.getZ();
            resRawZYX.nyOnLoading = nZnYnXOutput.getY();
            resRawZYX.nxOnLoading = nZnYnXOutput.getX();
            resRawZYX.setZYX(it, zyxFloatOutput);

            resRawZYX.maxValueT[it] = rawTZYX.maxValueT[it];
            resRawZYX.minValueT[it] = rawTZYX.minValueT[it];
            resRawZYX.maxValue = rawTZYX.maxValue;
            resRawZYX.minValue = rawTZYX.minValue;
            
            //<editor-fold defaultstate="collapsed" desc="debug: visualize result of interpolation">
        /*    SeriesModel series = FrameController.seriesRef.createNewSeriesCalc("OT", "SeriesInterp", 1);
            series.rawTZYX = resRawZYX;
            series.rawTZYX.updateMinMaxValues();
            ColorController.setDefaultWindowLevelFromHistogram(newSeries);
            FrameController.allFramesUpdate(newSeries);
            FrameController.allFramesUpdateBackgroundChanged();
*/
            //</editor-fold>
        }

        //<editor-fold defaultstate="collapsed" desc="log">
        msg += "\n"
                + " -> spacing: " + spatialResampling.toString() + " mm, "
                + " -> dim : nz" + resRawZYX.nz
                + " ny" + resRawZYX.ny
                + " nx" + resRawZYX.nx
                + " = tot of "
                + resRawZYX.nz
                * resRawZYX.ny
                * resRawZYX.nx + " voxels in volume"
                + "\n";
        FrameGui.LOGGER.log(Level.INFO, msg);
        //</editor-fold>

        return resRawZYX;
    }
}
