/*
 * USE OF THE LIFEx SOFTWARE IS SUBJECT TO THE FOLLOWING TERMS AND CONDITIONS. 
 * IF YOU DO NOT ACCEPT THESE TERMS AND CONDITIONS, DO NOT DOWNLOAD THE SOFTWARE. 
 * NON-PROFESSIONAL USERS ARE NOTIFIED THAT THE SOFTWARE AND THE RISKS INHERENT TO ITS USE,
 * MODIFICATION AND/OR DEVELOPMENT AND REPRODUCTION ARE NOT COVERED BY ANY WARRANTY ON THE PART OF THE CEA. 
 * IT IS HIGHLY RECOMMENDED THAT YOU ONLY LOAD AND/OR USE THE SOFTWARE IF YOU HAVE SPECIALIZED IN-DEPTH INFORMATION TECHNOLOGY KNOW-HOW.
 * THE COMMISSARIAT A L’ENERGIE ATOMIQUE ET AUX ENERGIES ALTERNATIVES (FRENCH ATOMIC ENERGY AGENCY), 
 * a public scientific, technical and industrial establishment, 
 * having its registered office at 25 rue Leblanc – bâtiment le Ponant D – 75015 Paris cedex, 
 * hereinafter referred to as "CEA", hereby licenses the right to use the LIFEx software as per the terms and conditions set out below.  
 */
package bardp.protocol.protocolTexture.gui;

import bardp.gui.FrameGui;
import bardp.img.controller.FrameController;
import bardp.protocol.ProtocolFrame;
import bardp.protocol.ProtocolList;
import bardp.protocol.protocolTexture.params.TextureJFrameOutputParameters;
import bardp.protocol.protocolTexture.textureGLCMMap.TextureGLCMMapController;
import bardp.protocol.protocolTexture.textureGLCMMap.TextureGLCMMapController.TexGLCMMapModel;
import bardp.session.patient.study.series.SeriesModel;
import bardp.utility.LIFExFonts;
import bardp.utility.MessageGui;
import bardp.utility.SvgAbstractButton;
import bardp.utility.WorkDir;
import bardp.utility.settings.SettingsOption;
import bardp.utility.swing.WrapFlowLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.util.Properties;
import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;

/**
 * <p>
 * Title: Local Image Feature Extraction (LIFEx)</p>
 * <p>
 * Description: </p>
 * <p>
 * Copyright: Copyright (c) 2016</p>
 * <p>
 * Company: Inserm/CNRS/CEA</p>
 * <p>
 * @author Christophe Nioche, Inserm at lifexsoft.org
 * @version 6.0
 */
public class TextureLocalMapJPanel extends JPanel {

    /**
     * Creates new form TexJPanelOption
     *
     */
    public TextureLocalMapJPanel() {
        this.prefsOption = new SettingsOption();

        initComponents();
        postComponents();

        texJFrameOutputParameters = new TextureJFrameOutputParameters();
    }

    public void setSeries(
            final SeriesModel series
    ) {
        this.series = series;

        //<editor-fold defaultstate="collapsed" desc="enable setting">
        this.jButtonIntensityDiscretizationDefaultSetting.setEnabled(false);
        this.jButtonIntensityResamplingDefaultSetting.setEnabled(false);

        boolean isPT_NM_CT = series.modality.equals("PT")
                || series.modality.equals("MN")
                || series.modality.equals("CT");
        this.jButtonIntensityDiscretizationDefaultSetting.setEnabled(isPT_NM_CT);
        this.jButtonIntensityResamplingDefaultSetting.setEnabled(isPT_NM_CT);
        //</editor-fold>

        this.filenameProperties
                = WorkDir.getRawVolumeSerieDirectory(series)
                + "TextureMap_Props.txt";

        setPropertiesValues();
    }

    private void postComponents() {

        this.setBackground(ProtocolList.TEXTURE.backgroundColorGui);
        this.setOpaque(true);
        
        this.setLayout(new WrapFlowLayout());
        this.jPanelInput.setLayout(new WrapFlowLayout());

        Border bGreyLine = BorderFactory.createLineBorder(Color.DARK_GRAY, 2, true);

        //<editor-fold defaultstate="collapsed" desc="IntensityResampling">
        this.jButtonIntensityResamplingDefaultSetting.setFont(LIFExFonts.FONT_SMALL);
        SvgAbstractButton.setIcon(jButtonIntensityResamplingDefaultSetting, "Default setting", SvgAbstractButton.TEXT_HIDE,
                "Set default settings", // toolTipText
                "SettingsDefault", FrameGui.svg.ICON_MINI_MINI_SIZE,
                SwingConstants.BOTTOM);

        SvgAbstractButton.setIcon(jButtonIntensityResamplingHelp, "Help", SvgAbstractButton.TEXT_HIDE,
                "<html>In PET, absolute resampling is preferred,<br>between 0 and the max SUV you expect in the VOI in your whole cohort.</html>", // toolTipText
                "Help", FrameGui.svg.ICON_MINI_MINI_SIZE,
                SwingConstants.BOTTOM);

        this.jTextFieldNbGrey.setFont(LIFExFonts.FONT_SMALL);
        this.jTextFieldBinSize.setFont(LIFExFonts.FONT_SMALL);
        this.jTextFieldMinBound.setFont(LIFExFonts.FONT_SMALL);
        this.jTextFieldMaxBound.setFont(LIFExFonts.FONT_SMALL);
        this.jRadioButtonAbsoluteMinMax.setFont(LIFExFonts.FONT_SMALL);
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="IntensityDiscretization">
        this.jPanelIntensityDiscretization.setBorder(BorderFactory.createTitledBorder(bGreyLine, "Intensity Discretization",
                javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
                javax.swing.border.TitledBorder.DEFAULT_POSITION,
                LIFExFonts.FONT_MEDIUM,
                Color.DARK_GRAY));
        SvgAbstractButton.setIcon(jButtonIntensityDiscretizationDefaultSetting, "Default setting", SvgAbstractButton.TEXT_HIDE,
                "Set default settings", // toolTipText
                "SettingsDefault", FrameGui.svg.ICON_MINI_MINI_SIZE,
                SwingConstants.BOTTOM);
        this.jButtonIntensityDiscretizationDefaultSetting.setFont(LIFExFonts.FONT_SMALL);
        this.jLabelBinSize.setFont(LIFExFonts.FONT_SMALL);
        this.jLabelNbGrey.setFont(LIFExFonts.FONT_SMALL);
        //</editor-fold>        

        //<editor-fold defaultstate="collapsed" desc="IntensityRescaling">
        this.jPanelIntensityRescaling.setBorder(BorderFactory.createTitledBorder(bGreyLine, "Intensity Rescaling",
                javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
                javax.swing.border.TitledBorder.DEFAULT_POSITION,
                LIFExFonts.FONT_MEDIUM,
                Color.DARK_GRAY));
        SvgAbstractButton.setIcon(jButtonIntensityDiscretizationHelp, "Help", SvgAbstractButton.TEXT_HIDE,
                "In PET, 64 levels are usually appropriate.", // toolTipText
                "Help", FrameGui.svg.ICON_MINI_MINI_SIZE,
                SwingConstants.BOTTOM);
        this.jLabelMaxBound.setFont(LIFExFonts.FONT_SMALL);
        this.jLabelMinBound.setFont(LIFExFonts.FONT_SMALL);
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="From Box/ROI/Img">
        this.jPanelLocalTextures.setBorder(BorderFactory.createTitledBorder(bGreyLine, "From box/ROI/Img",
                javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
                javax.swing.border.TitledBorder.DEFAULT_POSITION,
                LIFExFonts.FONT_MEDIUM,
                Color.DARK_GRAY));

        this.jRadioButtonLocalTexturesOnROI.setFont(LIFExFonts.FONT_MINI);
        this.jRadioButtonLocalTexturesOnWB.setFont(LIFExFonts.FONT_MINI);
        this.jRadioButtonLocalTexturesBoundingBoxOnROI.setFont(LIFExFonts.FONT_MINI);
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="Kernel Textures">
        this.jPanelKernel.setBorder(BorderFactory.createTitledBorder(bGreyLine, "Kernel parameters",
                javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
                javax.swing.border.TitledBorder.DEFAULT_POSITION,
                LIFExFonts.FONT_MEDIUM,
                Color.DARK_GRAY));

        this.jRadioButtonKernel3.setFont(LIFExFonts.FONT_MINI);
        this.jRadioButtonKernel5.setFont(LIFExFonts.FONT_MINI);
        this.jRadioButtonKernel7.setFont(LIFExFonts.FONT_MINI);
        //</editor-fold>

        this.jPanelCommand.setBackground(ProtocolList.TEXTURE.backgroundColorGui.darker());
        this.jPanelCommand.setOpaque(true);
        SvgAbstractButton.setIcon(this.jButtonRun, "Run", SvgAbstractButton.TEXT_VISIBLE,
                "Create maps of GLCM features", // toolTipText
                "Run", FrameGui.svg.ICON_MEDIUM_SIZE,
                SwingConstants.BOTTOM);

        //<editor-fold defaultstate="collapsed" desc="same sizeX for all optionbox">
        int sizeX = this.jPanelIntensityRescaling.getPreferredSize().width;
        sizeX = Math.max(sizeX, jPanelIntensityDiscretization.getPreferredSize().width);
        sizeX = Math.max(sizeX, jPanelKernel.getPreferredSize().width);
        sizeX = Math.max(sizeX, jPanelCommand.getPreferredSize().width);

        this.jPanelIntensityDiscretization.setPreferredSize(
                new Dimension(sizeX, jPanelIntensityDiscretization.getPreferredSize().height)
        );
        this.jPanelIntensityRescaling.setPreferredSize(
                new Dimension(sizeX, jPanelIntensityRescaling.getPreferredSize().height)
        );

        this.jPanelLocalTextures.setPreferredSize(
                new Dimension(sizeX, jPanelLocalTextures.getPreferredSize().height)
        );
        this.jPanelKernel.setPreferredSize(
                new Dimension(sizeX, jPanelKernel.getPreferredSize().height)
        );
        this.jPanelCommand.setPreferredSize(
                new Dimension(sizeX, jPanelCommand.getPreferredSize().height)
        );

        //</editor-fold>
    }

    /**
     * set fields with all default (=validated) values function of modalities
     */
    private void setIntensityResamplingDefaultValues() {

        switch (series.modality) {
            case "PT":
            case "MN":
                setIntensityResamplingDefaultValuesPT_MN();
                break;
            case "CT":
                setIntensityResamplingDefaultValuesCT();
                break;
            default:
                setIntensityResamplingDefaultValuesOther();
                break;
        }
    }

    public void setPropertiesLoadedValuesFromScripting(
            final Properties properties
    ) {
        scriptingCsv = properties.getProperty("LIFEx.texture.sessionCsv");

        nbGreyLevels = Float.parseFloat(properties.getProperty("LIFEx.texture.NbGrey"));
        binSize = Float.parseFloat(properties.getProperty("LIFEx.texture.BinSize"));
        SwingUtilities.invokeLater(() -> {
            this.jTextFieldNbGrey.setText(String.valueOf(nbGreyLevels));
            this.jTextFieldBinSize.setText(String.valueOf(binSize));
        });
        buttonAbsolute = Boolean.parseBoolean(properties.getProperty("LIFEx.texture.ButtonAbsolute"));
        this.jRadioButtonAbsoluteMinMax.setSelected(buttonAbsolute);
        if (buttonAbsolute) {
            minBound = Float.parseFloat(properties.getProperty("LIFEx.texture.MinBound"));
            maxBound = Float.parseFloat(properties.getProperty("LIFEx.texture.MaxBound"));
            SwingUtilities.invokeLater(() -> {
                this.jTextFieldMinBound.setText(String.valueOf(minBound));
                this.jTextFieldMaxBound.setText(String.valueOf(maxBound));
            });
        }
    }

    private void setAllFieldValues() {
        SwingUtilities.invokeLater(() -> {
            this.jTextFieldNbGrey.setText(nbGreyLevels == NO_DEF ? NO_DEF_STRING : String.valueOf(nbGreyLevels));
            this.jTextFieldNbGrey.setCaretPosition(0);

            this.jTextFieldBinSize.setText(binSize == NO_DEF ? NO_DEF_STRING : String.valueOf(binSize));
            this.jTextFieldBinSize.setCaretPosition(0);

            this.jTextFieldMinBound.setText(minBound == NO_DEF ? NO_DEF_STRING : String.valueOf(minBound));
            this.jTextFieldMinBound.setCaretPosition(0);

            this.jTextFieldMaxBound.setText(maxBound == NO_DEF ? NO_DEF_STRING : String.valueOf(maxBound));
            this.jTextFieldMaxBound.setCaretPosition(0);

            this.jRadioButtonAbsoluteMinMax.setSelected(buttonAbsolute);

            this.jRadioButtonLocalTexturesOnWB.setSelected(fromWB);
            this.jRadioButtonLocalTexturesOnROI.setSelected(fromROI);
            this.jRadioButtonLocalTexturesBoundingBoxOnROI.setSelected(fromBoundingBoxOnROI);

            this.jRadioButtonKernel3.setSelected(kernel3);
            this.jRadioButtonKernel5.setSelected(kernel5);
            this.jRadioButtonKernel7.setSelected(kernel7);
        });
    }

    /**
     * set parameter values with properties
     */
    private void setPropertiesValues() {

        boolean isLoadingPropertiesSuccess = loadProperties();
        if (!isLoadingPropertiesSuccess) {
            setIntensityResamplingDefaultValues();
        }

        setAllFieldValues();
    }

    /**
     * set default values to PT && NM modality
     */
    private void setIntensityResamplingDefaultValuesPT_MN() {
        this.nbGreyLevels = 64f;
        this.binSize = 20f / this.nbGreyLevels;
        this.minBound = 0f;
        this.maxBound = 20f;
        this.buttonAbsolute = true;
    }

    /**
     * set default values to CT modality
     */
    private void setIntensityResamplingDefaultValuesCT() {
        this.binSize = 10f;
        this.nbGreyLevels = 4000f / this.binSize;
        this.minBound = -1000f;
        this.maxBound = 3000f;
        this.buttonAbsolute = true;
    }

    /**
     * set default values to the other modalities
     */
    private void setIntensityResamplingDefaultValuesOther() {
        this.binSize = NO_DEF;
        this.nbGreyLevels = NO_DEF;
        this.minBound = NO_DEF;
        this.maxBound = NO_DEF;
        this.buttonAbsolute = true;
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        buttonGroupResampling = new javax.swing.ButtonGroup();
        buttonGroupLocalTextures = new javax.swing.ButtonGroup();
        buttonGroupKernel = new javax.swing.ButtonGroup();
        jPanelInput = new javax.swing.JPanel();
        jPanelIntensityDiscretization = new javax.swing.JPanel();
        jPanelDefaultSetting = new javax.swing.JPanel();
        jButtonIntensityDiscretizationHelp = new javax.swing.JButton();
        jButtonIntensityDiscretizationDefaultSetting = new javax.swing.JButton();
        jPanel3 = new javax.swing.JPanel();
        jPanel5 = new javax.swing.JPanel();
        jLabelNbGrey = new javax.swing.JLabel();
        jTextFieldNbGrey = new javax.swing.JTextField();
        jPanel6 = new javax.swing.JPanel();
        jLabelBinSize = new javax.swing.JLabel();
        jTextFieldBinSize = new javax.swing.JTextField();
        jPanelIntensityRescaling = new javax.swing.JPanel();
        jPanel7 = new javax.swing.JPanel();
        jPanelAbsolute = new javax.swing.JPanel();
        jRadioButtonAbsoluteMinMax = new javax.swing.JRadioButton();
        jPanelAbsoluteParamtersMinMax = new javax.swing.JPanel();
        jPanelIntensityResamplingDefaultSetting = new javax.swing.JPanel();
        jButtonIntensityResamplingHelp = new javax.swing.JButton();
        jButtonIntensityResamplingDefaultSetting = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        jPanelMinBound = new javax.swing.JPanel();
        jLabelMinBound = new javax.swing.JLabel();
        jTextFieldMinBound = new javax.swing.JTextField();
        jPanelMaxBound = new javax.swing.JPanel();
        jLabelMaxBound = new javax.swing.JLabel();
        jTextFieldMaxBound = new javax.swing.JTextField();
        jPanelRelativeRoiMinRoiMax = new javax.swing.JPanel();
        jPanelRelativeRoiSd = new javax.swing.JPanel();
        jPanelLocalTextures = new javax.swing.JPanel();
        jPanel8 = new javax.swing.JPanel();
        jPanel10 = new javax.swing.JPanel();
        jRadioButtonLocalTexturesOnWB = new javax.swing.JRadioButton();
        jPanel11 = new javax.swing.JPanel();
        jRadioButtonLocalTexturesOnROI = new javax.swing.JRadioButton();
        jPanel13 = new javax.swing.JPanel();
        jRadioButtonLocalTexturesBoundingBoxOnROI = new javax.swing.JRadioButton();
        jPanelKernel = new javax.swing.JPanel();
        jPanel12 = new javax.swing.JPanel();
        jPanel14 = new javax.swing.JPanel();
        jRadioButtonKernel3 = new javax.swing.JRadioButton();
        jPanel15 = new javax.swing.JPanel();
        jRadioButtonKernel5 = new javax.swing.JRadioButton();
        jPanel16 = new javax.swing.JPanel();
        jRadioButtonKernel7 = new javax.swing.JRadioButton();
        jPanelCommand = new javax.swing.JPanel();
        jButtonRun = new javax.swing.JButton();

        setOpaque(false);

        jPanelInput.setOpaque(false);

        jPanelIntensityDiscretization.setOpaque(false);
        jPanelIntensityDiscretization.setLayout(new java.awt.BorderLayout());

        jPanelDefaultSetting.setOpaque(false);
        jPanelDefaultSetting.setLayout(new javax.swing.BoxLayout(jPanelDefaultSetting, javax.swing.BoxLayout.Y_AXIS));

        jButtonIntensityDiscretizationHelp.setMargin(new java.awt.Insets(1, 1, 1, 1));
        jButtonIntensityDiscretizationHelp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonIntensityDiscretizationHelpActionPerformed(evt);
            }
        });
        jPanelDefaultSetting.add(jButtonIntensityDiscretizationHelp);

        jButtonIntensityDiscretizationDefaultSetting.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonIntensityDiscretizationDefaultSettingActionPerformed(evt);
            }
        });
        jPanelDefaultSetting.add(jButtonIntensityDiscretizationDefaultSetting);

        jPanelIntensityDiscretization.add(jPanelDefaultSetting, java.awt.BorderLayout.EAST);

        jPanel3.setOpaque(false);
        jPanel3.setLayout(new javax.swing.BoxLayout(jPanel3, javax.swing.BoxLayout.Y_AXIS));

        jPanel5.setOpaque(false);
        jPanel5.setLayout(new javax.swing.BoxLayout(jPanel5, javax.swing.BoxLayout.X_AXIS));

        jLabelNbGrey.setText("Nb of grey levels:   ");
        jPanel5.add(jLabelNbGrey);

        jTextFieldNbGrey.setColumns(5);
        jTextFieldNbGrey.setToolTipText("default: 64 bin");
        jTextFieldNbGrey.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                jTextFieldNbGreyKeyReleased(evt);
            }
        });
        jPanel5.add(jTextFieldNbGrey);

        jPanel3.add(jPanel5);

        jPanel6.setOpaque(false);
        jPanel6.setLayout(new javax.swing.BoxLayout(jPanel6, javax.swing.BoxLayout.X_AXIS));

        jLabelBinSize.setText("size of bins:  ");
        jPanel6.add(jLabelBinSize);

        jTextFieldBinSize.setColumns(5);
        jTextFieldBinSize.setToolTipText("default: 64 bin");
        jTextFieldBinSize.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                jTextFieldBinSizeKeyReleased(evt);
            }
        });
        jPanel6.add(jTextFieldBinSize);

        jPanel3.add(jPanel6);

        jPanelIntensityDiscretization.add(jPanel3, java.awt.BorderLayout.CENTER);

        jPanelInput.add(jPanelIntensityDiscretization);

        jPanelIntensityRescaling.setOpaque(false);
        jPanelIntensityRescaling.setLayout(new java.awt.BorderLayout());

        jPanel7.setOpaque(false);
        jPanel7.setLayout(new javax.swing.BoxLayout(jPanel7, javax.swing.BoxLayout.Y_AXIS));

        jPanelAbsolute.setOpaque(false);
        jPanelAbsolute.setLayout(new java.awt.BorderLayout());

        buttonGroupResampling.add(jRadioButtonAbsoluteMinMax);
        jRadioButtonAbsoluteMinMax.setSelected(true);
        jRadioButtonAbsoluteMinMax.setText("absolute (bounds: min<>max)");
        jRadioButtonAbsoluteMinMax.setToolTipText("get min and max bounds defined below");
        jRadioButtonAbsoluteMinMax.setContentAreaFilled(false);
        jRadioButtonAbsoluteMinMax.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                jRadioButtonAbsoluteMinMaxItemStateChanged(evt);
            }
        });
        jPanelAbsolute.add(jRadioButtonAbsoluteMinMax, java.awt.BorderLayout.CENTER);

        jPanel7.add(jPanelAbsolute);

        jPanelAbsoluteParamtersMinMax.setOpaque(false);
        jPanelAbsoluteParamtersMinMax.setLayout(new java.awt.BorderLayout());

        jPanelIntensityResamplingDefaultSetting.setOpaque(false);
        jPanelIntensityResamplingDefaultSetting.setLayout(new javax.swing.BoxLayout(jPanelIntensityResamplingDefaultSetting, javax.swing.BoxLayout.Y_AXIS));

        jButtonIntensityResamplingHelp.setMargin(new java.awt.Insets(1, 1, 1, 1));
        jButtonIntensityResamplingHelp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonIntensityResamplingHelpActionPerformed(evt);
            }
        });
        jPanelIntensityResamplingDefaultSetting.add(jButtonIntensityResamplingHelp);

        jButtonIntensityResamplingDefaultSetting.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonIntensityResamplingDefaultSettingActionPerformed(evt);
            }
        });
        jPanelIntensityResamplingDefaultSetting.add(jButtonIntensityResamplingDefaultSetting);

        jPanelAbsoluteParamtersMinMax.add(jPanelIntensityResamplingDefaultSetting, java.awt.BorderLayout.EAST);

        jPanel2.setOpaque(false);
        jPanel2.setLayout(new javax.swing.BoxLayout(jPanel2, javax.swing.BoxLayout.Y_AXIS));

        jPanelMinBound.setOpaque(false);
        jPanelMinBound.setLayout(new java.awt.BorderLayout());

        jLabelMinBound.setText("min bound:   ");
        jPanelMinBound.add(jLabelMinBound, java.awt.BorderLayout.WEST);

        jTextFieldMinBound.setColumns(5);
        jTextFieldMinBound.setToolTipText("default: 20");
        jTextFieldMinBound.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                jTextFieldMinBoundKeyReleased(evt);
            }
        });
        jPanelMinBound.add(jTextFieldMinBound, java.awt.BorderLayout.CENTER);

        jPanel2.add(jPanelMinBound);

        jPanelMaxBound.setOpaque(false);
        jPanelMaxBound.setLayout(new java.awt.BorderLayout());

        jLabelMaxBound.setText("max bound:   ");
        jPanelMaxBound.add(jLabelMaxBound, java.awt.BorderLayout.WEST);

        jTextFieldMaxBound.setColumns(5);
        jTextFieldMaxBound.setToolTipText("default: 20");
        jTextFieldMaxBound.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                jTextFieldMaxBoundKeyReleased(evt);
            }
        });
        jPanelMaxBound.add(jTextFieldMaxBound, java.awt.BorderLayout.CENTER);

        jPanel2.add(jPanelMaxBound);

        jPanelAbsoluteParamtersMinMax.add(jPanel2, java.awt.BorderLayout.CENTER);

        jPanel7.add(jPanelAbsoluteParamtersMinMax);

        jPanelRelativeRoiMinRoiMax.setOpaque(false);
        jPanelRelativeRoiMinRoiMax.setLayout(new java.awt.BorderLayout());
        jPanel7.add(jPanelRelativeRoiMinRoiMax);

        jPanelRelativeRoiSd.setOpaque(false);
        jPanelRelativeRoiSd.setLayout(new java.awt.BorderLayout());
        jPanel7.add(jPanelRelativeRoiSd);

        jPanelIntensityRescaling.add(jPanel7, java.awt.BorderLayout.CENTER);

        jPanelInput.add(jPanelIntensityRescaling);

        add(jPanelInput);

        jPanelLocalTextures.setOpaque(false);
        jPanelLocalTextures.setLayout(new java.awt.BorderLayout());

        jPanel8.setOpaque(false);
        jPanel8.setLayout(new javax.swing.BoxLayout(jPanel8, javax.swing.BoxLayout.Y_AXIS));

        jPanel10.setOpaque(false);
        jPanel10.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 5, 0));

        buttonGroupLocalTextures.add(jRadioButtonLocalTexturesOnWB);
        jRadioButtonLocalTexturesOnWB.setText("on whole body");
        jRadioButtonLocalTexturesOnWB.setContentAreaFilled(false);
        jRadioButtonLocalTexturesOnWB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonLocalTexturesOnWBActionPerformed(evt);
            }
        });
        jPanel10.add(jRadioButtonLocalTexturesOnWB);

        jPanel8.add(jPanel10);

        jPanel11.setOpaque(false);
        jPanel11.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 5, 0));

        buttonGroupLocalTextures.add(jRadioButtonLocalTexturesOnROI);
        jRadioButtonLocalTexturesOnROI.setSelected(true);
        jRadioButtonLocalTexturesOnROI.setText("on voxels from activated ROI");
        jRadioButtonLocalTexturesOnROI.setContentAreaFilled(false);
        jRadioButtonLocalTexturesOnROI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonLocalTexturesOnROIActionPerformed(evt);
            }
        });
        jPanel11.add(jRadioButtonLocalTexturesOnROI);

        jPanel8.add(jPanel11);

        jPanel13.setOpaque(false);
        jPanel13.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 5, 0));

        buttonGroupLocalTextures.add(jRadioButtonLocalTexturesBoundingBoxOnROI);
        jRadioButtonLocalTexturesBoundingBoxOnROI.setText("on bounding box of activated ROI");
        jRadioButtonLocalTexturesBoundingBoxOnROI.setContentAreaFilled(false);
        jRadioButtonLocalTexturesBoundingBoxOnROI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonLocalTexturesBoundingBoxOnROIActionPerformed(evt);
            }
        });
        jPanel13.add(jRadioButtonLocalTexturesBoundingBoxOnROI);

        jPanel8.add(jPanel13);

        jPanelLocalTextures.add(jPanel8, java.awt.BorderLayout.CENTER);

        add(jPanelLocalTextures);

        jPanelKernel.setOpaque(false);
        jPanelKernel.setLayout(new java.awt.BorderLayout());

        jPanel12.setOpaque(false);
        jPanel12.setLayout(new javax.swing.BoxLayout(jPanel12, javax.swing.BoxLayout.Y_AXIS));

        jPanel14.setOpaque(false);
        jPanel14.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 5, 0));

        buttonGroupKernel.add(jRadioButtonKernel3);
        jRadioButtonKernel3.setSelected(true);
        jRadioButtonKernel3.setText("kernel 3");
        jRadioButtonKernel3.setContentAreaFilled(false);
        jRadioButtonKernel3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonKernel3ActionPerformed(evt);
            }
        });
        jPanel14.add(jRadioButtonKernel3);

        jPanel12.add(jPanel14);

        jPanel15.setOpaque(false);
        jPanel15.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 5, 0));

        buttonGroupKernel.add(jRadioButtonKernel5);
        jRadioButtonKernel5.setText("kernel 5");
        jRadioButtonKernel5.setContentAreaFilled(false);
        jRadioButtonKernel5.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonKernel5ActionPerformed(evt);
            }
        });
        jPanel15.add(jRadioButtonKernel5);

        jPanel12.add(jPanel15);

        jPanel16.setOpaque(false);
        jPanel16.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 5, 0));

        buttonGroupKernel.add(jRadioButtonKernel7);
        jRadioButtonKernel7.setText("kernel 7");
        jRadioButtonKernel7.setContentAreaFilled(false);
        jRadioButtonKernel7.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonKernel7ActionPerformed(evt);
            }
        });
        jPanel16.add(jRadioButtonKernel7);

        jPanel12.add(jPanel16);

        jPanelKernel.add(jPanel12, java.awt.BorderLayout.CENTER);

        add(jPanelKernel);

        jButtonRun.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonRunActionPerformed(evt);
            }
        });
        jPanelCommand.add(jButtonRun);

        add(jPanelCommand);
    }// </editor-fold>//GEN-END:initComponents

    private void jTextFieldNbGreyKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTextFieldNbGreyKeyReleased
        try {
            maxBound = Float.parseFloat(this.jTextFieldMaxBound.getText());
            minBound = Float.parseFloat(this.jTextFieldMinBound.getText());
            nbGreyLevels = Float.parseFloat(this.jTextFieldNbGrey.getText());
            binSize = (maxBound - minBound) / (nbGreyLevels - 1);

            saveProperties();

            SwingUtilities.invokeLater(() -> {
                this.jTextFieldBinSize.setText(String.valueOf(binSize));
            });
        } catch (Exception ignore) {
        }

    }//GEN-LAST:event_jTextFieldNbGreyKeyReleased

    private void jTextFieldBinSizeKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTextFieldBinSizeKeyReleased
        try {
            maxBound = Float.parseFloat(this.jTextFieldMaxBound.getText());
            minBound = Float.parseFloat(this.jTextFieldMinBound.getText());
            binSize = Float.parseFloat(this.jTextFieldBinSize.getText());
            nbGreyLevels = (int) ((maxBound - minBound) / binSize) + 1;
            SwingUtilities.invokeLater(() -> {
                this.jTextFieldNbGrey.setText(String.valueOf(nbGreyLevels));
            });
            saveProperties();
        } catch (Exception ignore) {
        }
    }//GEN-LAST:event_jTextFieldBinSizeKeyReleased

    private void jTextFieldMinBoundKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTextFieldMinBoundKeyReleased
        try {
            maxBound = Float.parseFloat(this.jTextFieldMaxBound.getText());
            minBound = Float.parseFloat(this.jTextFieldMinBound.getText());

            if (minBound >= maxBound) {
                MessageGui messageGUI = new MessageGui(
                        "Texture message",
                        "Range of bounds is not valid.",
                        MessageGui.WARNING_MESSAGE);
            }
            nbGreyLevels = Float.parseFloat(this.jTextFieldNbGrey.getText());
            binSize = (maxBound - minBound) / nbGreyLevels;
            SwingUtilities.invokeLater(() -> {
                this.jTextFieldBinSize.setText(String.valueOf(binSize));
            });
            saveProperties();
        } catch (Exception ignore) {
        }
    }//GEN-LAST:event_jTextFieldMinBoundKeyReleased

    private void jTextFieldMaxBoundKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTextFieldMaxBoundKeyReleased
        try {
            maxBound = Float.parseFloat(this.jTextFieldMaxBound.getText());
            minBound = Float.parseFloat(this.jTextFieldMinBound.getText());
            if (minBound >= maxBound) {
                MessageGui messageGUI = new MessageGui(
                        "Texture message",
                        "Range of bounds is not valid.",
                        MessageGui.WARNING_MESSAGE);
            }
            nbGreyLevels = Float.parseFloat(this.jTextFieldNbGrey.getText());
            binSize = (maxBound - minBound) / nbGreyLevels;
            SwingUtilities.invokeLater(() -> {
                this.jTextFieldBinSize.setText(String.valueOf(binSize));
            });
            saveProperties();
        } catch (Exception ignore) {
        }
    }//GEN-LAST:event_jTextFieldMaxBoundKeyReleased

    private void jRadioButtonAbsoluteMinMaxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_jRadioButtonAbsoluteMinMaxItemStateChanged
        this.jLabelMinBound.setEnabled(true);
        this.jTextFieldMinBound.setEnabled(true);
        this.jLabelMaxBound.setEnabled(true);
        this.jTextFieldMaxBound.setEnabled(true);

        //    this.jLabelBinSize.setEnabled(true); // since v3.63
        //    this.jTextFieldBinSize.setEnabled(true);
        saveProperties();
    }//GEN-LAST:event_jRadioButtonAbsoluteMinMaxItemStateChanged

    private void jButtonIntensityDiscretizationDefaultSettingActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonIntensityDiscretizationDefaultSettingActionPerformed
        this.setIntensityResamplingDefaultValues();
        this.setAllFieldValues();
        MessageGui messageGUI = new MessageGui(
                "Texture message",
                "Default settings were applied",
                MessageGui.OK_MESSAGE
        );
        saveProperties();

    }//GEN-LAST:event_jButtonIntensityDiscretizationDefaultSettingActionPerformed

    private void jButtonIntensityResamplingDefaultSettingActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonIntensityResamplingDefaultSettingActionPerformed
        this.setIntensityResamplingDefaultValues();
        this.setAllFieldValues();
        MessageGui messageGUI = new MessageGui(
                "Texture message",
                "Default settings were applied",
                MessageGui.OK_MESSAGE
        );
        saveProperties();

    }//GEN-LAST:event_jButtonIntensityResamplingDefaultSettingActionPerformed

    private void jButtonIntensityResamplingHelpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonIntensityResamplingHelpActionPerformed
        MessageGui messageGUI = new MessageGui(
                "Intensity Rescaling",
                "In PET, AD is preferred, between 0 and the max SUV you expect in the VOI in your whole cohort.",
                MessageGui.OK_MESSAGE
        );
    }//GEN-LAST:event_jButtonIntensityResamplingHelpActionPerformed

    private void jButtonIntensityDiscretizationHelpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonIntensityDiscretizationHelpActionPerformed
        MessageGui messageGUI = new MessageGui(
                "Intensity Discretization",
                "In PET, 64 levels are usually appropriate.", // toolTipText
                MessageGui.OK_MESSAGE
        );

    }//GEN-LAST:event_jButtonIntensityDiscretizationHelpActionPerformed

   private void jRadioButtonLocalTexturesOnWBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonLocalTexturesOnWBActionPerformed
       if (this.jRadioButtonLocalTexturesOnWB.isSelected()) {
           MessageGui messageGUI = new MessageGui(
                   "Warning: long calculation time",
                   "Create maps of GLCM features on the whole image can take long.",
                   MessageGui.WARNING_MESSAGE
           );
       }
       saveProperties();

   }//GEN-LAST:event_jRadioButtonLocalTexturesOnWBActionPerformed

    private void jButtonRunActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonRunActionPerformed
        if (series == null) {
            if (FrameGui.session.getSeriesCount() == 1) {
                series = FrameController.seriesRef;
            } else {
                new MessageGui(
                        "Texture",
                        "No selected series found. Operation aborted",
                        MessageGui.WARNING_MESSAGE
                );
                return;
            }
        }

        if (FrameGui.protocolFrame.getTypeRepaint() == ProtocolFrame.HORIZONTAL_REPAINT) {
            FrameGui.protocolFrame.texBoxOption.setCloseOption();
        }
        FrameController.setActivatedSeriesOnAllFrames(series);

        TexGLCMMapModel model = this.jRadioButtonLocalTexturesOnROI.isSelected() ? TexGLCMMapModel.ON_ROI
                : this.jRadioButtonLocalTexturesBoundingBoxOnROI.isSelected() ? TexGLCMMapModel.ON_BOUNDING_BOX_ON_ROI
                : TexGLCMMapModel.ON_WHOLE_BODY;

        int kernelSize
                = this.jRadioButtonKernel3.isSelected() ? 3
                : this.jRadioButtonKernel5.isSelected() ? 5
                : this.jRadioButtonKernel7.isSelected() ? 7
                : 3;

        (new TextureGLCMMapController(
                series,
                kernelSize,
                model,
                this)).start();

    }//GEN-LAST:event_jButtonRunActionPerformed

    private void jRadioButtonKernel3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonKernel3ActionPerformed
        saveProperties();
    }//GEN-LAST:event_jRadioButtonKernel3ActionPerformed

    private void jRadioButtonKernel5ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonKernel5ActionPerformed
        saveProperties();
    }//GEN-LAST:event_jRadioButtonKernel5ActionPerformed

    private void jRadioButtonKernel7ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonKernel7ActionPerformed
        saveProperties();
    }//GEN-LAST:event_jRadioButtonKernel7ActionPerformed

    private void jRadioButtonLocalTexturesOnROIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonLocalTexturesOnROIActionPerformed
        saveProperties();
    }//GEN-LAST:event_jRadioButtonLocalTexturesOnROIActionPerformed

    private void jRadioButtonLocalTexturesBoundingBoxOnROIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonLocalTexturesBoundingBoxOnROIActionPerformed
        saveProperties();
    }//GEN-LAST:event_jRadioButtonLocalTexturesBoundingBoxOnROIActionPerformed

    public float getIntensityRescalingMaxBound() {
        return this.maxBound == NO_DEF ? 0f : this.maxBound;

    }

    public float getIntensityRescalingMinBound() {
        return this.minBound == NO_DEF ? 0f : this.minBound;

    }

    public float getIntensityDiscretizationNbGreyLevels() {
        return this.nbGreyLevels == NO_DEF ? 0f : this.nbGreyLevels;

    }

    public float getIntensityDiscretizationBinSize() {
        return this.binSize == NO_DEF ? 0f : this.binSize;

    }

    private void saveProperties() {
        prefsOption.setFloat("LIFEx.TextureMap.NbGrey", nbGreyLevels);
        prefsOption.setFloat("LIFEx.TextureMap.BinSize", binSize);

        prefsOption.setFloat("LIFEx.TextureMap.MinBound", minBound);
        prefsOption.setFloat("LIFEx.TextureMap.MaxBound", maxBound);

        prefsOption.setBoolean("LIFEx.TextureMap.ButtonAbsolute", buttonAbsolute);

        prefsOption.setBoolean("LIFEx.TextureMap.FromWB", fromWB);
        prefsOption.setBoolean("LIFEx.TextureMap.FromROI", fromROI);
        prefsOption.setBoolean("LIFEx.TextureMap.FromBoundingBoxOnROI", fromBoundingBoxOnROI);

        prefsOption.setBoolean("LIFEx.TextureMap.Kernel3", kernel3);
        prefsOption.setBoolean("LIFEx.TextureMap.Kernel5", kernel5);
        prefsOption.setBoolean("LIFEx.TextureMap.Kernel7", kernel7);

        prefsOption.saveProperties(filenameProperties);
    }

    /**
     * load properties from properties file named filenameProperties
     * <br>if file properties is not exist, setting values with default values
     *
     * @return true if loading success, false otherwise
     */
    private boolean loadProperties() {
        if (prefsOption.loadProperties(filenameProperties)) {

            nbGreyLevels = prefsOption.getFloat("LIFEx.TextureMap.NbGrey", NO_DEF);
            binSize = prefsOption.getFloat("LIFEx.TextureMap.BinSize", NO_DEF);

            minBound = prefsOption.getFloat("LIFEx.TextureMap.MinBound", NO_DEF);
            maxBound = prefsOption.getFloat("LIFEx.TextureMap.MaxBound", NO_DEF);

            buttonAbsolute = prefsOption.getBoolean("LIFEx.TextureMap.ButtonAbsolute", true);

            fromWB = prefsOption.getBoolean("LIFEx.TextureMap.FromWB", true);
            fromROI = prefsOption.getBoolean("LIFEx.TextureMap.FromROI", false);
            fromBoundingBoxOnROI = prefsOption.getBoolean("LIFEx.TextureMap.FromBoundingBoxOnROI", false);

            kernel3 = prefsOption.getBoolean("LIFEx.TextureMap.Kernel3", true);
            kernel5 = prefsOption.getBoolean("LIFEx.TextureMap.Kernel5", false);
            kernel7 = prefsOption.getBoolean("LIFEx.TextureMap.Kernel7", false);

            return true;
        } else {
            nbGreyLevels = NO_DEF;
            binSize = NO_DEF;

            minBound = NO_DEF;
            maxBound = NO_DEF;

            buttonAbsolute = true;

            fromWB = true;
            fromROI = false;
            fromBoundingBoxOnROI = false;

            kernel3 = true;
            kernel5 = false;
            kernel7 = false;

            return false;
        }
    }

    private float nbGreyLevels;
    private float binSize;

    private float minBound;
    private float maxBound;

    private boolean buttonAbsolute;

    private boolean fromWB;
    private boolean fromROI;
    private boolean fromBoundingBoxOnROI;

    private boolean kernel3;
    private boolean kernel5;
    private boolean kernel7;

    public String scriptingCsv = bardp.utility.settings.Settings.getString("LIFEx.TextureMap.SessionCsv", "");

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroupKernel;
    private javax.swing.ButtonGroup buttonGroupLocalTextures;
    private javax.swing.ButtonGroup buttonGroupResampling;
    private javax.swing.JButton jButtonIntensityDiscretizationDefaultSetting;
    private javax.swing.JButton jButtonIntensityDiscretizationHelp;
    private javax.swing.JButton jButtonIntensityResamplingDefaultSetting;
    private javax.swing.JButton jButtonIntensityResamplingHelp;
    public javax.swing.JButton jButtonRun;
    private javax.swing.JLabel jLabelBinSize;
    private javax.swing.JLabel jLabelMaxBound;
    private javax.swing.JLabel jLabelMinBound;
    private javax.swing.JLabel jLabelNbGrey;
    private javax.swing.JPanel jPanel10;
    private javax.swing.JPanel jPanel11;
    private javax.swing.JPanel jPanel12;
    private javax.swing.JPanel jPanel13;
    private javax.swing.JPanel jPanel14;
    private javax.swing.JPanel jPanel15;
    private javax.swing.JPanel jPanel16;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel7;
    private javax.swing.JPanel jPanel8;
    private javax.swing.JPanel jPanelAbsolute;
    private javax.swing.JPanel jPanelAbsoluteParamtersMinMax;
    private javax.swing.JPanel jPanelCommand;
    private javax.swing.JPanel jPanelDefaultSetting;
    private javax.swing.JPanel jPanelInput;
    private javax.swing.JPanel jPanelIntensityDiscretization;
    private javax.swing.JPanel jPanelIntensityResamplingDefaultSetting;
    private javax.swing.JPanel jPanelIntensityRescaling;
    private javax.swing.JPanel jPanelKernel;
    private javax.swing.JPanel jPanelLocalTextures;
    private javax.swing.JPanel jPanelMaxBound;
    private javax.swing.JPanel jPanelMinBound;
    private javax.swing.JPanel jPanelRelativeRoiMinRoiMax;
    private javax.swing.JPanel jPanelRelativeRoiSd;
    public javax.swing.JRadioButton jRadioButtonAbsoluteMinMax;
    private javax.swing.JRadioButton jRadioButtonKernel3;
    private javax.swing.JRadioButton jRadioButtonKernel5;
    private javax.swing.JRadioButton jRadioButtonKernel7;
    private javax.swing.JRadioButton jRadioButtonLocalTexturesBoundingBoxOnROI;
    private javax.swing.JRadioButton jRadioButtonLocalTexturesOnROI;
    private javax.swing.JRadioButton jRadioButtonLocalTexturesOnWB;
    public javax.swing.JTextField jTextFieldBinSize;
    public javax.swing.JTextField jTextFieldMaxBound;
    public javax.swing.JTextField jTextFieldMinBound;
    public javax.swing.JTextField jTextFieldNbGrey;
    // End of variables declaration//GEN-END:variables

    public TextureJFrameOutputParameters texJFrameOutputParameters;

    private SeriesModel series;
    private final SettingsOption prefsOption;
    private String filenameProperties;
    static private final float NO_DEF = -99999f;
    static private final String NO_DEF_STRING = "No def?";
}
