/*
 * USE OF THE LIFEx SOFTWARE IS SUBJECT TO THE FOLLOWING TERMS AND CONDITIONS. 
 * IF YOU DO NOT ACCEPT THESE TERMS AND CONDITIONS, DO NOT DOWNLOAD THE SOFTWARE. 
 * NON-PROFESSIONAL USERS ARE NOTIFIED THAT THE SOFTWARE AND THE RISKS INHERENT TO ITS USE,
 * MODIFICATION AND/OR DEVELOPMENT AND REPRODUCTION ARE NOT COVERED BY ANY WARRANTY ON THE PART OF THE CEA. 
 * IT IS HIGHLY RECOMMENDED THAT YOU ONLY LOAD AND/OR USE THE SOFTWARE IF YOU HAVE SPECIALIZED IN-DEPTH INFORMATION TECHNOLOGY KNOW-HOW.
 * THE COMMISSARIAT A L’ENERGIE ATOMIQUE ET AUX ENERGIES ALTERNATIVES (FRENCH ATOMIC ENERGY AGENCY), 
 * a public scientific, technical and industrial establishment, 
 * having its registered office at 25 rue Leblanc – bâtiment le Ponant D – 75015 Paris cedex, 
 * hereinafter referred to as "CEA", hereby licenses the right to use the LIFEx software as per the terms and conditions set out below.  
 */
package bardp.protocol.protocolTexture.histo;

import bardp.gui.FrameGui;
import bardp.img.controller.FrameController;
import bardp.roi.controller.RoiModel;
import bardp.utility.MessageGui;
import bardp.utility.Utils;
import bardp.utility.WorkDir;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.logging.Level;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFDataFormat;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFPalette;
import org.apache.poi.hssf.usermodel.HSSFPrintSetup;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.FillPatternType;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.jfree.data.statistics.HistogramDataset;

/**
 * <p>
 * Title: Local Image Feature Extraction (LIFEx)</p>
 * <p>
 * Description: </p>
 * <p>
 * Copyright: Copyright (c) 2016</p>
 * <p>
 * Company: Inserm/CNRS/CEA</p>
 * <p>
 * @author Christophe Nioche, Inserm at lifexsoft.org
 * @version 6.0
 */
public class HistogramIO_xls extends Thread {

   public HistogramIO_xls(
           final HistogramDataset histogramDataset,
           final RoiModel roiModel
   ) {
      this.histogramDataset = histogramDataset;
      this.roiModel = roiModel;
   }

   @Override
   public void run() {

      //<editor-fold defaultstate="collapsed" desc="check texture process">
      if (roiModel.texControllerSub == null) {
         MessageGui messageGUI = new MessageGui(
                 "Texture Protocol",
                 "No texture or no ROI found. Create it and try again.",
                 MessageGui.ERROR_MESSAGE);
         return;
      }
      //</editor-fold>

      //<editor-fold defaultstate="collapsed" desc="texturePatientFile">
      String roiDirectory = WorkDir.getROIValueDirectory(FrameController.seriesRef);
      bardp.utility.Utils.createDirectory(roiDirectory);

      long time = System.currentTimeMillis();
      String sortfilename = FrameController.seriesRef.fromStudy.fromPatient.patientName.trim().toUpperCase()
              + "_" + FrameController.seriesRef.fromStudy.fromPatient.patientID.trim().toUpperCase()
              + "_" + roiModel.title
              + "_Histogram";
      sortfilename = sortfilename.replace("\\", "_");
      sortfilename = sortfilename.replaceAll("/", "_");

      String strfile = Utils.prepDirectoryAndFile(roiDirectory, sortfilename
              + "_" + time + ".xls");

      File histogramPatientFile = new File(strfile);
      createNewFileXls(histogramPatientFile);
      //</editor-fold>

   }

   public void createNewFileXls(
           final File file
   ) {

      try (FileOutputStream fileOut = new FileOutputStream(file)) {

         workbook = new HSSFWorkbook();
         createFont();

         for (int iSeries = 0; iSeries < histogramDataset.getSeriesCount(); iSeries++) {

            //<editor-fold defaultstate="collapsed" desc="sheet creation">
            Comparable roiName = histogramDataset.getSeriesKey(iSeries);

            String nameSheet = FrameGui.SOFTNAME + FrameGui.VERSION_SHORT
                    + "_" + roiName.toString();

            // 31 cf. https://poi.apache.org/apidocs/org/apache/poi/hssf/usermodel/HSSFWorkbook.html
            if (nameSheet.length() > 31) {
               nameSheet = "_RoiNameTooLong_" + iSeries;
            }

            HSSFSheet sheet = workbook.createSheet(nameSheet);
            //</editor-fold>

            saveInfoToSeries(sheet, iSeries);
         }
         workbook.write(fileOut);
      } catch (IOException ex) {
         FrameGui.LOGGER.log(Level.SEVERE,
                 ex.toString() + "File: {0} [  error  ] ", file.getName()
         );
      }

      MessageGui messageGUI = new MessageGui("Creation of XL file [ OK ]", file.getName(), MessageGui.OK_MESSAGE);
      FrameGui.LOGGER.log(Level.INFO, "File :{0} [  OK  ]", file.getName());
   }

   private void createFont() {
      // get the workbook custom pallete and set the color at the seed roiIndex to the custom color
      HSSFPalette palette = workbook.getCustomPalette();

      short colorIndexTabTitle = (short) (HSSFColor.HSSFColorPredefined.BLACK.getIndex() + 2);
      palette.setColorAtIndex(colorIndexTabTitle, (byte) 206, (byte) 206, (byte) 206);

      short colorIndexValue = (short) (HSSFColor.HSSFColorPredefined.BLACK.getIndex() + 3);
      palette.setColorAtIndex(colorIndexValue, (byte) 231, (byte) 231, (byte) 231);

      // Tab Unit
      HSSFFont fontTabUnit = workbook.createFont();
      fontTabUnit.setFontHeightInPoints((short) 12);
      fontTabUnit.setFontName("Arial");
      styleTabUnit = workbook.createCellStyle();
      styleTabUnit.setFont(fontTabUnit);
      styleTabUnit.setAlignment(HorizontalAlignment.CENTER);
      styleTabUnit.setBorderTop(BorderStyle.THIN);
      styleTabUnit.setTopBorderColor(HSSFColor.HSSFColorPredefined.BLACK.getIndex());
      styleTabUnit.setFillForegroundColor(colorIndexTabTitle);
      styleTabUnit.setFillPattern(FillPatternType.SOLID_FOREGROUND);
      styleTabUnit.setVerticalAlignment(VerticalAlignment.CENTER);
      styleTabUnit.setWrapText(true);

      // Value
      HSSFFont fontValue = workbook.createFont();
      fontValue.setFontHeightInPoints((short) 8);
      fontValue.setFontName("Courier");

      HSSFDataFormat df = workbook.createDataFormat();

      styleValue0deci = workbook.createCellStyle();
      styleValue0deci.setFont(fontValue);
      styleValue0deci.setFillForegroundColor(colorIndexValue);
       styleValue0deci.setFillPattern(FillPatternType.SOLID_FOREGROUND);
      styleValue0deci.setVerticalAlignment(VerticalAlignment.CENTER);
      styleValue0deci.setDataFormat(df.getFormat("#0"));

      styleValue5deci = workbook.createCellStyle();
      styleValue5deci.setFont(fontValue);
      styleValue5deci.setFillForegroundColor(colorIndexValue);
      styleValue5deci.setFillPattern(FillPatternType.SOLID_FOREGROUND);
      styleValue5deci.setVerticalAlignment(VerticalAlignment.CENTER);
      styleValue5deci.setDataFormat(df.getFormat("#0.0000"));
   }

   private void saveInfoToSeries(
           final HSSFSheet sheet,
           final int iSeries
   ) {

      short sizeHeightTile = 500;
      int iRow = 0;
      int iColumn = 0;
      HSSFCell cell;
      HSSFRow row;

      // <editor-fold defaultstate="collapsed" desc="Sheet parameters & size width of columns">
      sheet.setDisplayGridlines(false);
      sheet.setDefaultColumnWidth(10);

      sheet.setGridsPrinted(false);
      sheet.setPrintGridlines(false);
      sheet.getPrintSetup().setPaperSize(HSSFPrintSetup.A4_PAPERSIZE);
      sheet.setZoom(3, 4); // = 75%
      // </editor-fold>

      //<editor-fold defaultstate="collapsed" desc="all titles">
      row = sheet.createRow(iRow);
      row.setHeight((short) sizeHeightTile);

      //<editor-fold defaultstate="collapsed" desc="PatientName title & ROI title">
      iColumn = 0;
      cell = row.createCell(iColumn);
      cell.setCellValue("x histo");
      cell.setCellStyle(styleTabUnit);

      iColumn++;
      cell = row.createCell(iColumn);
      cell.setCellValue("# histo");
      cell.setCellStyle(styleTabUnit);

      iColumn--;
      iColumn--;
      //</editor-fold>

      double x, y;
      for (int item = 0; item < histogramDataset.getItemCount(iSeries); item++) {

         iRow++;
         row = sheet.createRow(iRow);

         iColumn++;
         cell = row.createCell(iColumn);
         //x = histogramDataset.getEndXValue(iSeries, item);
         x = histogramDataset.getStartXValue(iSeries, item);
         cell.setCellValue(x);
         cell.setCellStyle(styleValue5deci);

         iColumn++;
         cell = row.createCell(iColumn);
         y = histogramDataset.getYValue(iSeries, item);
         cell.setCellValue(y);
         cell.setCellStyle(styleValue0deci);

         iColumn--;
         iColumn--;
      }

      int sheetIndex = 0;
      workbook.setPrintArea(sheetIndex, 1, 2, 1, iRow);
   }

   private HSSFWorkbook workbook;
   private HSSFCellStyle styleTabUnit;
   private HSSFCellStyle styleValue5deci;
   private HSSFCellStyle styleValue0deci;

   private final HistogramDataset histogramDataset;
   private final RoiModel roiModel;
}
