/*
 * USE OF THE LIFEx SOFTWARE IS SUBJECT TO THE FOLLOWING TERMS AND CONDITIONS. 
 * IF YOU DO NOT ACCEPT THESE TERMS AND CONDITIONS, DO NOT DOWNLOAD THE SOFTWARE. 
 * NON-PROFESSIONAL USERS ARE NOTIFIED THAT THE SOFTWARE AND THE RISKS INHERENT TO ITS USE,
 * MODIFICATION AND/OR DEVELOPMENT AND REPRODUCTION ARE NOT COVERED BY ANY WARRANTY ON THE PART OF THE CEA. 
 * IT IS HIGHLY RECOMMENDED THAT YOU ONLY LOAD AND/OR USE THE SOFTWARE IF YOU HAVE SPECIALIZED IN-DEPTH INFORMATION TECHNOLOGY KNOW-HOW.
 * THE COMMISSARIAT A L’ENERGIE ATOMIQUE ET AUX ENERGIES ALTERNATIVES (FRENCH ATOMIC ENERGY AGENCY), 
 * a public scientific, technical and industrial establishment, 
 * having its registered office at 25 rue Leblanc – bâtiment le Ponant D – 75015 Paris cedex, 
 * hereinafter referred to as "CEA", hereby licenses the right to use the LIFEx software as per the terms and conditions set out below.  
 */
package bardp.protocol.protocolTexture.io;

import bardp.gui.FrameGui;
import bardp.gui.FrameGui.ExecutionModel;
import bardp.session.patient.study.series.SeriesModel;
import bardp.img.controller.FrameController;
import bardp.img.picImage.PicImageBoxOptionController;
import bardp.img.gui.PicImageBoxOption;
import bardp.session.patient.study.series.SeriesUnits;
import bardp.roi.io.RoiLoadFile;
import bardp.roi.controller.RoiMain;
import bardp.protocol.protocolTexture.TextureController;
import bardp.session.patient.study.series.load.SeriesLoading;
import bardp.roi.controller.RoiController;
import bardp.roi.controller.RoiMainList;
import bardp.utility.MessageGui;
import bardp.utility.Utils;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Properties;
import java.util.logging.Level;

/**
 * <p>
 * Title: Local Image Feature Extraction (LIFEx)</p>
 * <p>
 * Description: </p>
 * <p>
 * Copyright: Copyright (c) 2016</p>
 * <p>
 * Company: Inserm/CNRS/CEA</p>
 * <p>
 * @author Christophe Nioche, Inserm at lifexsoft.org
 * @version 6.0
 */
public class TextureMultiSeriesScripting {

    /**
     * files: many script files is agree
     *
     * @param files
     * @param frameGUI
     */
    public TextureMultiSeriesScripting(
            File[] files,
            FrameGui frameGUI
    ) {
        this.files = files;
        this.frameGUI = frameGUI;
    }

    public byte run() {

        for (File file : files) {

            if (file.isDirectory()) {
                return ERROR;
            }
            //<editor-fold defaultstate="collapsed" desc="load / check properties file">
            Properties properties;
            try {
                properties = load(file);
            } catch (Exception ex) {
                return ERROR;
            }
            if (properties == null) {
                return ERROR;
            }
            // only for quickly exit if no DICOMDIR file
            if (!properties.containsKey("LIFEx.texture.Session0.Img0")) {
                return ERROR;
            }
            if (checkCompleteProperties(properties) == ERROR) {
                return ERROR;
            }

            //</editor-fold>
            // set executionModel to EXEC_SCRIPTING_TEXTURE
            boolean exitAtEnd = FrameGui.getExecutionModel() == ExecutionModel.EXEC_SCRIPTING_TEXTURE_ARGUMENT;

            if (FrameGui.getExecutionModel() == ExecutionModel.EXEC_NORMAL
                    || FrameGui.getExecutionModel() == ExecutionModel.EXEC_SCRIPTING_TEXTURE_ARGUMENT) {
                FrameGui.setExecutionModel(ExecutionModel.EXEC_SCRIPTING_TEXTURE);
            }

            //<editor-fold defaultstate="collapsed" desc="log: starting script">
            FrameGui.LOGGER.log(
                    Level.INFO,
                    "\n"
                    + "############################################\n"
                    + "Script starting of file:" + file.getName() + "\n"
                    + "############################################\n"
            );
            //</editor-fold>

            int sessionIndex = 0;
            String sessionKey = "LIFEx.texture.Session" + sessionIndex;
            while (properties.containsKey(sessionKey + ".Img0")) {

                //<editor-fold defaultstate="collapsed" desc="log session">
                FrameGui.LOGGER.log(
                        Level.INFO,
                        "\n"
                        + "############################################\n"
                        + "Session starting:" + sessionKey + "\n"
                        + "############################################\n"
                );
                //</editor-fold>

                //<editor-fold defaultstate="collapsed" desc="loading img">
                int imgIndex = 0;
                String imgKey = sessionKey + ".Img" + imgIndex;
                while (properties.containsKey(imgKey)) {
                    String imgFile = properties.getProperty(imgKey, "null").trim();
                    //<editor-fold defaultstate="collapsed" desc="log: img loading">
                    FrameGui.LOGGER.log(
                            Level.INFO,
                            "\n"
                            + "     _________________________________________________\n"
                            + "\n"
                            + "        " + imgKey + " " + imgFile + " ... [  loading  ]\n"
                            + "     _________________________________________________\n"
                    );
                    //</editor-fold>          
                    File[] seriesFiles = new File[]{
                        new File(imgFile)
                    };

                    SeriesLoading seriesLoad = new SeriesLoading(
                            frameGUI,
                            seriesFiles,
                            false, true,
                            null, null
                    );
                    seriesLoad.run();

                    imgKey = sessionKey + ".Img" + ++imgIndex;
                }
                //</editor-fold>

                Utils.waitUntilEventQueueIsEmpty();

                // check unitY of all series loading
                for (imgIndex = 0; imgIndex < FrameGui.session.getSeriesCount(); imgIndex++) {

                    imgKey = sessionKey + ".Img" + imgIndex;
                    if (properties.containsKey(imgKey + ".unitY")) {

                        SeriesModel series = FrameGui.session.getAllSeries().get(imgIndex);

                        String unitY = properties.getProperty(imgKey + ".unitY");
                        if (unitY.equals(SeriesUnits.UNIT_SUV_BW.getShortDescription())) {
                            series.seriesUnit.unitY = SeriesUnits.UNIT_SUV_BW;
                        } else if (unitY.equals(SeriesUnits.UNIT_SUV_LBM.getShortDescription())) {
                            series.seriesUnit.unitY = SeriesUnits.UNIT_SUV_LBM;
                        } else if (unitY.equals(SeriesUnits.UNIT_kBqmL.getShortDescription())) {
                            series.seriesUnit.unitY = SeriesUnits.UNIT_kBqmL;
                        } else if (unitY.equals(SeriesUnits.UNIT_CpxVx.getShortDescription())) {
                            series.seriesUnit.unitY = SeriesUnits.UNIT_CpxVx;
                        }

                        PicImageBoxOption pibo = PicImageBoxOptionController.getPicImageBoxOptionFromSerie(series);
                        pibo.picImageJPanelOption.curveJPanelOption.setUnitY(series.seriesUnit.unitY);
                    }
                }

                //<editor-fold defaultstate="collapsed" desc="loading ROI">
                int roiIndex = 0;
                String roiKey = sessionKey + ".Roi" + roiIndex;
                while (properties.containsKey(roiKey)) {
                    String roiFile = properties.getProperty(roiKey, "null").trim();
                    //<editor-fold defaultstate="collapsed" desc="log: loading roi">
                    FrameGui.LOGGER.log(
                            Level.INFO,
                            "\n"
                            + "             *******************************************************\n"
                            + "             " + roiKey + " " + roiFile + " ... [  loading  ]\n"
                            + "             *******************************************************\n"
                    );
                    //</editor-fold>
                    RoiLoadFile.loadRoiFiles(new File[]{new File(roiFile)},
                            RoiMain.WITH_WAITING
                    );

                    Utils.waitUntilEventQueueIsEmpty();

                    //<editor-fold defaultstate="collapsed" desc="delete structure not wanted (only for RTStruct file)">
                    StringBuilder msgRtStruct = new StringBuilder("RTStruct:: selection of structures  [  check  ]\n");
                    // first step, get wanted structures on script file
                    int rtStructIndex = 0;
                    String rtStructKey = roiKey + "RTStruct" + rtStructIndex;
                    if (properties.containsKey(rtStructKey)) { // if check if need
                        ArrayList<String> nameOfStructureWanted = new ArrayList<>();
                        while (properties.containsKey(rtStructKey)) {
                            nameOfStructureWanted.add(properties.getProperty(rtStructKey, "null").trim());
                            rtStructIndex++;
                            rtStructKey = roiKey + "RTStruct" + rtStructIndex;
                        }
                        // second step, get all loaded structures not wanted in roiMainToDelete
                        RoiMainList roiMainToDelete = new RoiMainList();
                        for (RoiMain roiMain : RoiController.roiMainList) {
                            boolean isDelete = true;
                            for (int ii = 0; ii < nameOfStructureWanted.size() && isDelete; ii++) {
                                String name = nameOfStructureWanted.get(ii);
                                if (roiMain.roiModel.title.contains(name)) {
                                    msgRtStruct.append("RTStruct::").append(roiMain.roiModel.title).append("  [  keep  ]\n");
                                    isDelete = false;
                                } else {
                                    msgRtStruct.append("RTStruct::").append(roiMain.roiModel.title).append("  [  delete  ]\n");
                                    isDelete = true;
                                }
                            }
                            if (isDelete) {
                                roiMainToDelete.add(roiMain);
                            }

                        }
                        // third step, remove all structure in roiMainToDelete
                        for (RoiMain roiMain : roiMainToDelete) {
                            RoiController.disposeRoi(roiMain);
                        }
                        Utils.waitUntilEventQueueIsEmpty();
                    }
                    FrameGui.LOGGER.log(
                            Level.INFO,
                            msgRtStruct.toString()
                    );
                    //</editor-fold>

                    FrameGui.LOGGER.log(
                            Level.INFO,
                            "\n"
                            + "             *******************************************************\n"
                            + "             " + roiKey + " " + roiFile + " ... [  end calculating  ]\n"
                            + "             *******************************************************\n"
                    );
                    roiKey = sessionKey + ".Roi" + ++roiIndex;
                }
                //</editor-fold>

                Utils.waitUntilEventQueueIsEmpty();

                //<editor-fold defaultstate="collapsed" desc="running calculation of texture">
                int _imgIndex = 0;
                for (SeriesModel series : FrameGui.session.getAllSeries()) {
                    FrameController.setActivatedSeriesOnAllFrames(series);

                    (new TextureController(series, properties, sessionKey + ".Img" + _imgIndex)).run();
                    _imgIndex++;
                }
                //</editor-fold>

                //<editor-fold defaultstate="collapsed" desc="log: end calculating">
                FrameGui.LOGGER.log(
                        Level.INFO,
                        "\n"
                        + "     _________________________________________________\n"
                        + "\n"
                        + "         ... [  end session calculating  ]\n"
                        + "     _________________________________________________\n"
                );
                //</editor-fold>

                FrameGui.session.removePatient(FrameController.seriesRef.fromStudy.fromPatient);
                System.gc();
                sessionKey = "LIFEx.texture.Session" + ++sessionIndex;
            }

            //<editor-fold defaultstate="collapsed" desc="log: Script ending">
            FrameGui.LOGGER.log(
                    Level.INFO,
                    "\n"
                    + "############################################\n"
                    + "Script ending of file: {0}", file.getName() + "\n"
                    + "############################################\n"
            );
            //</editor-fold>

            if (exitAtEnd) {
                System.exit(0);
            }
        }
        FrameGui.setExecutionModel(ExecutionModel.EXEC_NORMAL);
        return OK;
    }

    /**
     * check to complete Properties file
     *
     * @param properties
     * @return
     */
    private int checkCompleteProperties(
            final Properties properties
    ) {

        FrameGui.LOGGER.log(Level.INFO, "Script check... [  start  ]");

        //<editor-fold defaultstate="collapsed" desc="check is exist properties">
        if (properties == null) {
            String msg = "Properties not found";
            MessageGui messageGUI = new MessageGui(
                    "Error message",
                    msg,
                    MessageGui.ERROR_MESSAGE);
            FrameGui.LOGGER.log(Level.INFO, msg);
            return ERROR;
        }
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="check is properties isEmpty">
        if (properties.isEmpty()) {
            String msg = "Properties is Empty";
            MessageGui messageGUI = new MessageGui(
                    "Error message",
                    msg,
                    MessageGui.ERROR_MESSAGE);
            FrameGui.LOGGER.log(Level.INFO, msg);
            return ERROR;
        }
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="check if all minimal mandatory Keys is in properties">
        final String allKeyProperties[] = new String[]{
            "LIFEx.texture.BinSize",
            "LIFEx.texture.NbGrey",
            "LIFEx.texture.SessionCsv", // new start
            "LIFEx.texture.ButtonAbsolute",
            // "LIFEx.texture.MinBound", // not necessary if relative option choosed
            // "LIFEx.texture.MaxBound",
            "LIFEx.texture.ButtonRelativeMeanSd",
            "LIFEx.texture.ButtonRelativeMinMax",
            //"LIFEx.texture.GLCM.DistanceWithNeighbours", // default 1 vx if not present
            "LIFEx.texture.Session0.Img0",
            "LIFEx.texture.Session0.Roi0"
        };

        for (String key : allKeyProperties) {
            if (!properties.containsKey(key)) {
                MessageGui messageGUI = new MessageGui(
                        "Error message",
                        key + " not found in Properties",
                        MessageGui.ERROR_MESSAGE);
                FrameGui.LOGGER.log(Level.INFO,
                        "{0} not found in Properties", key);
                return ERROR;
            }
        }
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="check is only one button is checked (true)">
        int nbButtonTrue
                = (properties.getProperty("LIFEx.texture.ButtonAbsolute").trim().equals("true") ? 1 : 0)
                + (properties.getProperty("LIFEx.texture.ButtonRelativeMeanSd").trim().equals("true") ? 1 : 0)
                + (properties.getProperty("LIFEx.texture.ButtonRelativeMinMax").trim().equals("true") ? 1 : 0);
        if (nbButtonTrue != 1) {
            String msg = "More than one test (button) is activated (true). "
                    + "Active a single test among these possible choices:\n"
                    + "- ButtonAbsolute\n"
                    + "- ButtonRelativeMeanSd\n"
                    + "- ButtonRelativeMinMax";
            MessageGui messageGUI = new MessageGui(
                    "Error message", msg,
                    MessageGui.ERROR_MESSAGE
            );
            FrameGui.LOGGER.log(Level.INFO, msg);
            return ERROR;
        }
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="check distanceWithNeighbours">
        try {
            int distanceWithNeighbours = Integer.parseInt(properties.getProperty("LIFEx.texture.GLCM.DistanceWithNeighbours").trim());
            if (distanceWithNeighbours < 1) {
                String msg = "The lower value (distanceWithNeighbours) must be greater or equal to 1.\n"
                        + "Correct it and try again";
                MessageGui messageGUI = new MessageGui(
                        "Error message", msg,
                        MessageGui.ERROR_MESSAGE);
                FrameGui.LOGGER.log(Level.INFO, msg);
                return ERROR;
            }
            if (distanceWithNeighbours > 10) {
                String msg = "The upper value (distanceWithNeighbours) must be lower or equal to 10.\n"
                        + "Correct it and try again";
                MessageGui messageGUI = new MessageGui(
                        "Error message", msg,
                        MessageGui.ERROR_MESSAGE);
                FrameGui.LOGGER.log(Level.INFO, msg);
                return ERROR;
            }
        } catch (Exception ignore) { // if not present then default value of gui is got
        }
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="check is min < max whn ButtonRelativeMinMax is checked">
        if (properties.getProperty("LIFEx.texture.ButtonAbsolute").trim().equals("true")) {
            float minBound, maxBound;
            //<editor-fold defaultstate="collapsed" desc="check 'number' value of minBound">
            try {
                minBound = Float.parseFloat(properties.getProperty("LIFEx.texture.MinBound").trim());
            } catch (NumberFormatException ex) {
                String msg = "The lower bound (minBound) isn't a number.\n"
                        + "Correct it and try again";
                MessageGui messageGUI = new MessageGui(
                        "Error message", msg,
                        MessageGui.ERROR_MESSAGE);
                FrameGui.LOGGER.log(Level.INFO, msg);
                return ERROR;
            }
            //</editor-fold>
            //</editor-fold>
            //</editor-fold>
            //</editor-fold>

            //<editor-fold defaultstate="collapsed" desc="check 'number' value of maxBound">
            try {
                maxBound = Float.parseFloat(properties.getProperty("LIFEx.texture.MaxBound").trim());
            } catch (NumberFormatException ex) {
                String msg = "The upper bound (maxBound) isn't a number.\n"
                        + "Correct it and try again";
                MessageGui messageGUI = new MessageGui(
                        "Error message", msg,
                        MessageGui.ERROR_MESSAGE);
                FrameGui.LOGGER.log(Level.INFO, msg);
                return ERROR;
            }
            //</editor-fold>
            //</editor-fold>
            //</editor-fold>
            //</editor-fold>

            //<editor-fold defaultstate="collapsed" desc="check minBound < maxBound">
            if (minBound >= maxBound) {
                String msg = "The lower bound (minBound) is larger than the upper bound (maxBound)\n"
                        + "Correct it and try again";
                MessageGui messageGUI = new MessageGui(
                        "Error message", msg,
                        MessageGui.ERROR_MESSAGE);
                FrameGui.LOGGER.log(Level.INFO, msg);
                return ERROR;
            }
            //</editor-fold>
        }
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="check 'number' value of BinSize">
        try {
            Float value = Float.parseFloat(properties.getProperty("LIFEx.texture.BinSize").trim());
        } catch (NumberFormatException ex) {
            String msg = "The BinSize isn't a number.\n"
                    + "Correct it and try again";
            MessageGui messageGUI = new MessageGui(
                    "Error message", msg,
                    MessageGui.ERROR_MESSAGE);
            FrameGui.LOGGER.log(Level.INFO, msg);
            return ERROR;
        }
        //</editor-fold>
        //</editor-fold>
        //</editor-fold>
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="check 'number' value of NbGrey">
        try {
            Float value = Float.parseFloat(properties.getProperty("LIFEx.texture.NbGrey").trim());
        } catch (NumberFormatException ex) {
            String msg = "The NbGrey isn't a number.\n"
                    + "Correct it and try again";
            MessageGui messageGUI = new MessageGui(
                    "Error message", msg,
                    MessageGui.ERROR_MESSAGE);
            FrameGui.LOGGER.log(Level.INFO, msg);
            return ERROR;
        }
        //</editor-fold>
        //</editor-fold>
        //</editor-fold>
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="LIFEx.texture.Img Roi">
        //try {
        int sessionIndex = 0;
        String file;
        String sessionKey = "LIFEx.texture.Session" + sessionIndex;
        while (properties.containsKey(sessionKey + ".Img0")) {

            //<editor-fold defaultstate="collapsed" desc="loading Img">
            int imgIndex = 0;
            String imgKey = sessionKey + ".Img" + imgIndex;
            while (properties.containsKey(imgKey)) {

                file = properties.getProperty(imgKey).trim();
                boolean isFile = new File(file).exists();
                if (!isFile) {
                    String msg = imgKey + " isn't a valid file or directory.\n"
                            + "Correct it and try again";
                    MessageGui messageGUI = new MessageGui(
                            "Error message", msg,
                            MessageGui.ERROR_MESSAGE
                    );
                    FrameGui.LOGGER.log(Level.INFO, msg);
                    return ERROR;
                }

                imgKey = sessionKey + ".Img" + ++imgIndex;
            }
            //</editor-fold>

            //<editor-fold defaultstate="collapsed" desc="loading Roi">
            int roiIndex = 0;
            String roiKey = sessionKey + ".Roi" + roiIndex;
            while (properties.containsKey(roiKey)) {

                file = properties.getProperty(roiKey).trim();
                boolean isFile = new File(file).exists();
                if (!isFile) {
                    String msg = roiKey + " isn't a valid file or directory.\n"
                            + "Correct it and try again";
                    MessageGui messageGUI = new MessageGui(
                            "Error message", msg,
                            MessageGui.ERROR_MESSAGE
                    );
                    FrameGui.LOGGER.log(Level.INFO, msg);
                    return ERROR;
                }

                roiKey = sessionKey + ".Roi" + ++roiIndex;
            }
            //</editor-fold>

            sessionKey = "LIFEx.texture.Session" + ++sessionIndex;
        }

        FrameGui.LOGGER.log(Level.INFO, "Script check... [  end  ]");
        FrameGui.setExecutionModel(ExecutionModel.EXEC_NORMAL);

        //</editor-fold>
        return OK;
    }

    /**
     * Finds and loads properties scripting file
     *
     * @param file
     */
    private Properties load(
            final File file
    ) {
        Properties properties = null;
        InputStream f;
        try {
            f = new BufferedInputStream(new FileInputStream(file));
            try {
                properties = new Properties();
                properties.load(f);
                f.close();
            } catch (IOException e) {
            }
        } catch (FileNotFoundException ex) {
            FrameGui.LOGGER.log(Level.SEVERE, ex.toString());
        }
        return properties;
    }

    static public final byte OK = 0;
    static public final byte ERROR = 1;

    /**
     * properties file
     */
    private final File[] files;
    private final FrameGui frameGUI;
}
