/*
 * USE OF THE LIFEx SOFTWARE IS SUBJECT TO THE FOLLOWING TERMS AND CONDITIONS. 
 * IF YOU DO NOT ACCEPT THESE TERMS AND CONDITIONS, DO NOT DOWNLOAD THE SOFTWARE. 
 * NON-PROFESSIONAL USERS ARE NOTIFIED THAT THE SOFTWARE AND THE RISKS INHERENT TO ITS USE,
 * MODIFICATION AND/OR DEVELOPMENT AND REPRODUCTION ARE NOT COVERED BY ANY WARRANTY ON THE PART OF THE CEA. 
 * IT IS HIGHLY RECOMMENDED THAT YOU ONLY LOAD AND/OR USE THE SOFTWARE IF YOU HAVE SPECIALIZED IN-DEPTH INFORMATION TECHNOLOGY KNOW-HOW.
 * THE COMMISSARIAT A L’ENERGIE ATOMIQUE ET AUX ENERGIES ALTERNATIVES (FRENCH ATOMIC ENERGY AGENCY), 
 * a public scientific, technical and industrial establishment, 
 * having its registered office at 25 rue Leblanc – bâtiment le Ponant D – 75015 Paris cedex, 
 * hereinafter referred to as "CEA", hereby licenses the right to use the LIFEx software as per the terms and conditions set out below.  
 */
package bardp.protocol.protocolTexture.params;

import bardp.protocol.ProtocolList;
import bardp.utility.LIFExFonts;
import bardp.utility.settings.Settings;
import bardp.utility.swing.JLabelRotate;
import java.awt.Color;
import javax.swing.BorderFactory;
import javax.swing.border.Border;

/**
 * <p>
 * Title: Local Image Feature Extraction (LIFEx)</p>
 * <p>
 * Description: </p>
 * <p>
 * Copyright: Copyright (c) 2016</p>
 * <p>
 * Company: Inserm/CNRS/CEA</p>
 * <p>
 * @author Christophe Nioche, Inserm at lifexsoft.org
 * @version 6.0
 */
public class TextureJPanelOutputParameters extends javax.swing.JPanel {

    /**
     * Creates new form TexJPanelOption
     */
    public TextureJPanelOutputParameters() {
        initComponents();
        postComponents();
        setProperties();
    }

    public final void postComponents() {

        Color backgroundColor = ProtocolList.TEXTURE.backgroundColorGui;

        jPanelFirstOrder.setBackground(backgroundColor.darker());
        jPanelSecondOrder.setBackground(backgroundColor);
        jPanelResults.setBackground(backgroundColor.darker());
                
        Color bGreyTitle = new Color(100, 100, 100);
        Border bGreyLine = BorderFactory.createLineBorder(bGreyTitle, 2, true);

        this.jLabelRotateFirstOrder.setRotation(JLabelRotate.ROTATE_LEFT);
        //<editor-fold defaultstate="collapsed" desc="jPanelSphericity">
        this.jPanelShape.setBorder(javax.swing.BorderFactory.createTitledBorder(bGreyLine, "Shape",
                javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
                javax.swing.border.TitledBorder.DEFAULT_POSITION,
                LIFExFonts.FONT_MEDIUM,
                bGreyTitle));
        jCheckBoxShape_Sphericity.setEnabled(true);
        jCheckBoxShape_Sphericity.setSelected(true);
        jCheckBoxShape_Compacity.setEnabled(true);
        jCheckBoxShape_Compacity.setSelected(true);
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="jPanelHistogram">
        this.jPanelHistogram.setBorder(javax.swing.BorderFactory.createTitledBorder(bGreyLine, "Histogram",
                javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
                javax.swing.border.TitledBorder.DEFAULT_POSITION,
                LIFExFonts.FONT_MEDIUM,
                bGreyTitle));
        jCheckBoxHistogram_Skewness.setEnabled(true);
        jCheckBoxHistogram_Skewness.setSelected(true);
        jCheckBoxHistogram_Kurtosis.setEnabled(true);
        jCheckBoxHistogram_Kurtosis.setSelected(true);
        jCheckBoxHistogram_Entropy_log10.setEnabled(true);
        jCheckBoxHistogram_Entropy_log10.setSelected(true);
        jCheckBoxHistogram_Energy.setEnabled(true);
        jCheckBoxHistogram_Energy.setSelected(true);
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="jPanelConventional">
        this.jPanelConventional.setBorder(javax.swing.BorderFactory.createTitledBorder(bGreyLine, "Conventional/Discretized",
                javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
                javax.swing.border.TitledBorder.DEFAULT_POSITION,
                LIFExFonts.FONT_MEDIUM,
                bGreyTitle));
        jCheckBoxConventional_SUVmin_mean_max.setEnabled(true);
        jCheckBoxConventional_Q1_Q2_Q3.setSelected(true);
        jCheckBoxConventional_SUVmin_mean_max.setSelected(true);
        jCheckBoxConventional_SUVpeakSphere05mL.setEnabled(true);
        jCheckBoxConventional_SUVpeakSphere05mL.setSelected(true);
        jCheckBoxConventional_SUVpeakSphere1mL.setEnabled(true);
        jCheckBoxConventional_SUVpeakSphere1mL.setSelected(true);
        jCheckBoxConventional_CalciumAgatstonScore.setSelected(true);
        jCheckBoxShape_Volume_mL.setEnabled(true);
        jCheckBoxShape_Volume_mL.setSelected(true);
        jCheckBoxConventional_TLGmL.setEnabled(true);
        jCheckBoxConventional_TLGmL.setSelected(true);
        //</editor-fold>

        this.jLabelRotateSecondOrder.setRotation(JLabelRotate.ROTATE_LEFT);
        //<editor-fold defaultstate="collapsed" desc="jPanelGLCM">
        this.jPanelGLCM.setBorder(javax.swing.BorderFactory.createTitledBorder(bGreyLine, "GLCM",
                javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
                javax.swing.border.TitledBorder.DEFAULT_POSITION,
                LIFExFonts.FONT_MEDIUM,
                bGreyTitle));

        jCheckBoxGLCM_Save.setSelected(false);
        jCheckBoxGLCM_Homogeneity.setEnabled(true);
        jCheckBoxGLCM_Homogeneity.setSelected(true);
        jCheckBoxGLCM_Energy.setEnabled(true);
        jCheckBoxGLCM_Energy.setSelected(true);
        jCheckBoxGLCM_Contrast.setEnabled(true);
        jCheckBoxGLCM_Contrast.setSelected(true);
        jCheckBoxGLCM_Correlation.setEnabled(true);
        jCheckBoxGLCM_Correlation.setSelected(true);
        jCheckBoxGLCM_Entropy.setEnabled(true);
        jCheckBoxGLCM_Entropy.setSelected(true);
        jCheckBoxGLCM_Dissimilarity.setEnabled(true);
        jCheckBoxGLCM_Dissimilarity.setSelected(true);
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="jPanelGLRM">
        this.jPanelGLRM.setBorder(javax.swing.BorderFactory.createTitledBorder(bGreyLine, "GLRM",
                javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
                javax.swing.border.TitledBorder.DEFAULT_POSITION,
                LIFExFonts.FONT_MEDIUM,
                bGreyTitle));

        jCheckBoxGLRLM_Save.setSelected(false);
        jCheckBoxGLRLM_SRE.setEnabled(true);
        jCheckBoxGLRLM_SRE.setSelected(true);
        jCheckBoxGLRLM_LRE.setEnabled(true);
        jCheckBoxGLRLM_LRE.setSelected(true);
        jCheckBoxGLRLM_LGRE.setEnabled(true);
        jCheckBoxGLRLM_LGRE.setSelected(true);
        jCheckBoxGLRLM_HGRE.setEnabled(true);
        jCheckBoxGLRLM_HGRE.setSelected(true);
        jCheckBoxGLRLM_SRLGE.setEnabled(true);
        jCheckBoxGLRLM_SRLGE.setSelected(true);
        jCheckBoxGLRLM_SRHGE.setEnabled(true);
        jCheckBoxGLRLM_SRHGE.setSelected(true);
        jCheckBoxGLRLM_LRLGE.setEnabled(true);
        jCheckBoxGLRLM_LRLGE.setSelected(true);
        jCheckBoxGLRLM_LRHGE.setEnabled(true);
        jCheckBoxGLRLM_LRHGE.setSelected(true);
        jCheckBoxGLRLM_GLNU.setEnabled(true);
        jCheckBoxGLRLM_GLNU.setSelected(true);
        jCheckBoxGLRLM_RLNU.setEnabled(true);
        jCheckBoxGLRLM_RLNU.setSelected(true);
        jCheckBoxGLRLM_RP.setEnabled(true);
        jCheckBoxGLRLM_RP.setSelected(true);
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="jPanelNGLDM">
        this.jPanelNGLDM.setBorder(javax.swing.BorderFactory.createTitledBorder(bGreyLine, "NGLDM",
                javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
                javax.swing.border.TitledBorder.DEFAULT_POSITION,
                LIFExFonts.FONT_MEDIUM,
                bGreyTitle));
        jCheckBoxNGLDM_Save.setSelected(false);
        jCheckBoxNGLDM_Coarseness.setEnabled(true);
        jCheckBoxNGLDM_Coarseness.setSelected(true);
        jCheckBoxNGLDM_Contrast.setEnabled(true);
        jCheckBoxNGLDM_Contrast.setSelected(true);
        jCheckBoxNGLDM_Busyness.setEnabled(true);
        jCheckBoxNGLDM_Busyness.setSelected(true);
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="jPanelGLZLM">
        this.jPanelGLZLM.setBorder(javax.swing.BorderFactory.createTitledBorder(bGreyLine, "GLZLM",
                javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
                javax.swing.border.TitledBorder.DEFAULT_POSITION,
                LIFExFonts.FONT_MEDIUM,
                bGreyTitle));
        jCheckBoxGLZLM_Save.setSelected(false);
        jCheckBoxGLZLM_SZE.setEnabled(true);
        jCheckBoxGLZLM_SZE.setSelected(true);
        jCheckBoxGLZLM_LZE.setEnabled(true);
        jCheckBoxGLZLM_LZE.setSelected(true);
        jCheckBoxGLZLM_LGZE.setEnabled(true);
        jCheckBoxGLZLM_LGZE.setSelected(true);
        jCheckBoxGLZLM_HGZE.setEnabled(true);
        jCheckBoxGLZLM_HGZE.setSelected(true);
        jCheckBoxGLZLM_SZLGE.setEnabled(true);
        jCheckBoxGLZLM_SZLGE.setSelected(true);
        jCheckBoxGLZLM_SZHGE.setEnabled(true);
        jCheckBoxGLZLM_SZHGE.setSelected(true);
        jCheckBoxGLZLM_LZLGE.setEnabled(true);
        jCheckBoxGLZLM_LZLGE.setSelected(true);
        jCheckBoxGLZLM_LZHGE.setEnabled(true);
        jCheckBoxGLZLM_LZHGE.setSelected(true);
        jCheckBoxGLZLM_GLNU.setEnabled(true);
        jCheckBoxGLZLM_GLNU.setSelected(true);
        jCheckBoxGLZLM_ZLNU.setEnabled(true);
        jCheckBoxGLZLM_ZLNU.setSelected(true);
        jCheckBoxGLZLM_ZP.setEnabled(true);
        jCheckBoxGLZLM_ZP.setSelected(true);
        //</editor-fold>

        this.jLabelRotateResults.setRotation(JLabelRotate.ROTATE_LEFT);
        

        //<editor-fold defaultstate="collapsed" desc="Histogram Output">
        this.jPanelHistogramFile.setBorder(javax.swing.BorderFactory.createTitledBorder(bGreyLine, "Histogram Output",
                javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
                javax.swing.border.TitledBorder.DEFAULT_POSITION,
                LIFExFonts.FONT_MEDIUM,
                bGreyTitle));
        //</editor-fold>

        
        //<editor-fold defaultstate="collapsed" desc="Matrix Output">
        this.jPanelMatrixFile.setBorder(javax.swing.BorderFactory.createTitledBorder(bGreyLine, "Matrix Output",
                javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
                javax.swing.border.TitledBorder.DEFAULT_POSITION,
                LIFExFonts.FONT_MEDIUM,
                bGreyTitle));
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="font">
        jLabelRotateFirstOrder.setFont(LIFExFonts.FONT_LARGE);

        jLabelDistance.setFont(LIFExFonts.FONT_SMALL);
        jLabelDistance1.setFont(LIFExFonts.FONT_SMALL);
        jComboBoxDistance.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLCM_Save.setFont(LIFExFonts.FONT_SMALL);
        jLabelInEachDirection.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLCM_Homogeneity.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLCM_Energy.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLCM_Contrast.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLCM_Correlation.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLCM_Entropy.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLCM_Dissimilarity.setFont(LIFExFonts.FONT_SMALL);

        jCheckBoxGLRLM_Save.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLRLM_SRE.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLRLM_LRE.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLRLM_LGRE.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLRLM_HGRE.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLRLM_SRLGE.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLRLM_SRHGE.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLRLM_LRLGE.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLRLM_LRHGE.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLRLM_GLNU.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLRLM_RLNU.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLRLM_RP.setFont(LIFExFonts.FONT_SMALL);

        jCheckBoxNGLDM_Save.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxNGLDM_Coarseness.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxNGLDM_Contrast.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxNGLDM_Busyness.setFont(LIFExFonts.FONT_SMALL);

        jCheckBoxGLZLM_Save.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLZLM_SZE.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLZLM_LZE.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLZLM_LGZE.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLZLM_HGZE.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLZLM_SZLGE.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLZLM_SZHGE.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLZLM_LZLGE.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLZLM_LZHGE.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLZLM_GLNU.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLZLM_ZLNU.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxGLZLM_ZP.setFont(LIFExFonts.FONT_SMALL);

        jLabelRotateSecondOrder.setFont(LIFExFonts.FONT_LARGE);

        jCheckBoxShape_Sphericity.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxShape_Compacity.setFont(LIFExFonts.FONT_SMALL);

        jCheckBoxHistogram_Skewness.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxHistogram_Kurtosis.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxHistogram_Entropy_log10.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxHistogram_Entropy_log2.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxHistogram_Energy.setFont(LIFExFonts.FONT_SMALL);

        jCheckBoxConventional_SUVmin_mean_max.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxConventional_SUVpeakSphere05mL.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxConventional_SUVpeakSphere1mL.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxConventional_CalciumAgatstonScore.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxShape_Volume_mL.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxShape_Volume_vx.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxConventional_TLGmL.setFont(LIFExFonts.FONT_SMALL);
        jCheckBoxConventional_RIMmin_mean_max.setFont(LIFExFonts.FONT_SMALL);

        jLabelRotateResults.setFont(LIFExFonts.FONT_LARGE);
        //</editor-fold>
    }

    private void setProperties() {        
        jCheckBoxConventional_SUVmin_mean_max.setSelected(Settings.getBoolean("LIFEx.texture.CONVENTIONAL.isSUVmin_mean_max", true));
        jCheckBoxConventional_Q1_Q2_Q3.setSelected(Settings.getBoolean("LIFEx.texture.CONVENTIONAL.isQ1_Q2_Q3", true));
        jCheckBoxConventional_SUVpeakSphere05mL.setSelected(Settings.getBoolean("LIFEx.texture.CONVENTIONAL.isSUVpeakSphere05mL", true));
        jCheckBoxConventional_SUVpeakSphere1mL.setSelected(Settings.getBoolean("LIFEx.texture.CONVENTIONAL.isSUVpeakSphere1mL", true));
        jCheckBoxConventional_CalciumAgatstonScore.setSelected(Settings.getBoolean("LIFEx.texture.CONVENTIONAL.isCalciumAgatstonScore", true));
        jCheckBoxConventional_TLGmL.setSelected(Settings.getBoolean("LIFEx.texture.CONVENTIONAL.isTLGmL", true));
        jCheckBoxConventional_RIMmin_mean_max.setSelected(Settings.getBoolean("LIFEx.texture.CONVENTIONAL.isRIM", true));
        
        jCheckBoxShape_Volume_mL.setSelected(Settings.getBoolean("LIFEx.texture.SHAPE.isVolume_mL", true));
        jCheckBoxShape_Volume_vx.setSelected(Settings.getBoolean("LIFEx.texture.SHAPE.isVolume_vx", true));
        jCheckBoxShape_Compacity.setSelected(Settings.getBoolean("LIFEx.texture.SHAPE.isCompacity", true));
        jCheckBoxShape_Sphericity.setSelected(Settings.getBoolean("LIFEx.texture.SHAPE.isSphericity", true));
        
        jCheckBoxHistogram_Energy.setSelected(Settings.getBoolean("LIFEx.texture.HISTO.isEnergy", true));
        jCheckBoxHistogram_Entropy_log10.setSelected(Settings.getBoolean("LIFEx.texture.HISTO.isEntropyLog10", true));
        jCheckBoxHistogram_Entropy_log2.setSelected(Settings.getBoolean("LIFEx.texture.HISTO.isEntropyLog2", true));
        jCheckBoxHistogram_Kurtosis.setSelected(Settings.getBoolean("LIFEx.texture.HISTO.isKurtosis", true));
        jCheckBoxHistogram_Skewness.setSelected(Settings.getBoolean("LIFEx.texture.HISTO.isSkewness", true));
        jCheckBoxHISTO_Show.setSelected(Settings.getBoolean("LIFEx.texture.HISTO.isShow", false));
        jCheckBoxHISTO_Save.setSelected(Settings.getBoolean("LIFEx.texture.HISTO.isSave", false));

        jCheckBoxGLCM_Contrast.setSelected(Settings.getBoolean("LIFEx.texture.GLCM.isContrast", true));
        jCheckBoxGLCM_Correlation.setSelected(Settings.getBoolean("LIFEx.texture.GLCM.isCorrelation", true));
        jCheckBoxGLCM_Dissimilarity.setSelected(Settings.getBoolean("LIFEx.texture.GLCM.isDissimilarity", true));
        jCheckBoxGLCM_Energy.setSelected(Settings.getBoolean("LIFEx.texture.GLCM.isEnergy", true));
        jCheckBoxGLCM_Entropy.setSelected(Settings.getBoolean("LIFEx.texture.GLCM.isEntropy", true));
        jCheckBoxGLCM_Homogeneity.setSelected(Settings.getBoolean("LIFEx.texture.GLCM.isHomogeneity", true));
        jCheckBoxGLCM_Save.setSelected(Settings.getBoolean("LIFEx.texture.GLCM.isSave", false));
        jCheckBoxGLCM_Contrast.setSelected(Settings.getBoolean("LIFEx.texture.GLCM.isContrast", true));
        jComboBoxDistance.setSelectedIndex(Settings.getInt("LIFEx.texture.GLCM.distanceWithNeighbours", 1)-1);
        
        jCheckBoxGLRLM_GLNU.setSelected(Settings.getBoolean("LIFEx.texture.GLRLM.isGLNU", true));
        jCheckBoxGLRLM_HGRE.setSelected(Settings.getBoolean("LIFEx.texture.GLRLM.isHGRE", true));
        jCheckBoxGLRLM_LGRE.setSelected(Settings.getBoolean("LIFEx.texture.GLRLM.isLGRE", true));
        jCheckBoxGLRLM_LRE.setSelected(Settings.getBoolean("LIFEx.texture.GLRLM.isLRE", true));
        jCheckBoxGLRLM_LRHGE.setSelected(Settings.getBoolean("LIFEx.texture.GLRLM.isLRHGE", true));
        jCheckBoxGLRLM_LRLGE.setSelected(Settings.getBoolean("LIFEx.texture.GLRLM.isLRLGE", true));
        jCheckBoxGLRLM_RLNU.setSelected(Settings.getBoolean("LIFEx.texture.GLRLM.isRLNU", true));
        jCheckBoxGLRLM_RP.setSelected(Settings.getBoolean("LIFEx.texture.GLRLM.isRP", true));
        jCheckBoxGLRLM_SRE.setSelected(Settings.getBoolean("LIFEx.texture.GLRLM.isSRE", true));
        jCheckBoxGLRLM_SRHGE.setSelected(Settings.getBoolean("LIFEx.texture.GLRLM.isSRHGE", true));
        jCheckBoxGLRLM_SRLGE.setSelected(Settings.getBoolean("LIFEx.texture.GLRLM.isSRLG", true));
        jCheckBoxGLRLM_Save.setSelected(Settings.getBoolean("LIFEx.texture.GLRLM.isSave", false));

        jCheckBoxGLZLM_GLNU.setSelected(Settings.getBoolean("LIFEx.texture.GLZLM.isGLNU", true));
        jCheckBoxGLZLM_HGZE.setSelected(Settings.getBoolean("LIFEx.texture.GLZLM.isHGZE", true));
        jCheckBoxGLZLM_LGZE.setSelected(Settings.getBoolean("LIFEx.texture.GLZLM.isLGZE", true));
        jCheckBoxGLZLM_LZE.setSelected(Settings.getBoolean("LIFEx.texture.GLZLM.isLZE", true));
        jCheckBoxGLZLM_LZHGE.setSelected(Settings.getBoolean("LIFEx.texture.GLZLM.isLZHGE", true));
        jCheckBoxGLZLM_LZLGE.setSelected(Settings.getBoolean("LIFEx.texture.GLZLM.isLZLGE", true));
        jCheckBoxGLZLM_SZE.setSelected(Settings.getBoolean("LIFEx.texture.GLZLM.isSZE", true));
        jCheckBoxGLZLM_SZHGE.setSelected(Settings.getBoolean("LIFEx.texture.GLZLM.isSZHGE", true));
        jCheckBoxGLZLM_SZLGE.setSelected(Settings.getBoolean("LIFEx.texture.GLZLM.isSZLGE", true));
        jCheckBoxGLZLM_Save.setSelected(Settings.getBoolean("LIFEx.texture.GLZLM.isSave", false));
        jCheckBoxGLZLM_ZLNU.setSelected(Settings.getBoolean("LIFEx.texture.GLZLM.isZLNU", true));
        jCheckBoxGLZLM_ZP.setSelected(Settings.getBoolean("LIFEx.texture.GLZLM.isZP", true));
        
        jCheckBoxNGLDM_Busyness.setSelected(Settings.getBoolean("LIFEx.texture.NGLDM.isBusyness", true));
        jCheckBoxNGLDM_Coarseness.setSelected(Settings.getBoolean("LIFEx.texture.NGLDM.isCoarseness", true));
        jCheckBoxNGLDM_Contrast.setSelected(Settings.getBoolean("LIFEx.texture.NGLDM.isContrast", true));
        jCheckBoxNGLDM_Save.setSelected(Settings.getBoolean("LIFEx.texture.NGLDM.isSave", false));
    }

    protected void saveProperties() {        
        Settings.setBoolean("LIFEx.texture.CONVENTIONAL.isSUVmin_mean_max", jCheckBoxConventional_SUVmin_mean_max.isSelected());
        Settings.setBoolean("LIFEx.texture.CONVENTIONAL.isQ1_Q2_Q3", jCheckBoxConventional_Q1_Q2_Q3.isSelected());
        Settings.setBoolean("LIFEx.texture.CONVENTIONAL.isSUVpeakSphere05mL", jCheckBoxConventional_SUVpeakSphere05mL.isSelected());
        Settings.setBoolean("LIFEx.texture.CONVENTIONAL.isSUVpeakSphere1mL", jCheckBoxConventional_SUVpeakSphere1mL.isSelected());
        Settings.setBoolean("LIFEx.texture.CONVENTIONAL.isCalciumAgatstonScore", jCheckBoxConventional_CalciumAgatstonScore.isSelected());
        Settings.setBoolean("LIFEx.texture.CONVENTIONAL.isTLGmL", jCheckBoxConventional_TLGmL.isSelected());
        Settings.setBoolean("LIFEx.texture.CONVENTIONAL.isRIM", jCheckBoxConventional_RIMmin_mean_max.isSelected());
        
        Settings.setBoolean("LIFEx.texture.SHAPE.isVolume_vx", jCheckBoxShape_Volume_vx.isSelected());
        Settings.setBoolean("LIFEx.texture.SHAPE.isVolume_mL", jCheckBoxShape_Volume_mL.isSelected());
        Settings.setBoolean("LIFEx.texture.SHAPE.isCompacity", jCheckBoxShape_Compacity.isSelected());
        Settings.setBoolean("LIFEx.texture.SHAPE.isSphericity", jCheckBoxShape_Sphericity.isSelected());
        
        Settings.setBoolean("LIFEx.texture.HISTO.isShow", jCheckBoxHISTO_Show.isSelected());
        Settings.setBoolean("LIFEx.texture.HISTO.isSave", jCheckBoxHISTO_Save.isSelected());
        Settings.setBoolean("LIFEx.texture.HISTO.isEnergy", jCheckBoxHistogram_Energy.isSelected());
        Settings.setBoolean("LIFEx.texture.HISTO.isEntropyLog10", jCheckBoxHistogram_Entropy_log10.isSelected());
        Settings.setBoolean("LIFEx.texture.HISTO.isEntropyLog2", jCheckBoxHistogram_Entropy_log2.isSelected());
        Settings.setBoolean("LIFEx.texture.HISTO.isKurtosis", jCheckBoxHistogram_Kurtosis.isSelected());
        Settings.setBoolean("LIFEx.texture.HISTO.isSkewness", jCheckBoxHistogram_Skewness.isSelected());
        
        Settings.setBoolean("LIFEx.texture.GLCM.isContrast", jCheckBoxGLCM_Contrast.isSelected());
        Settings.setBoolean("LIFEx.texture.GLCM.isCorrelation", jCheckBoxGLCM_Correlation.isSelected());
        Settings.setBoolean("LIFEx.texture.GLCM.isDissimilarity", jCheckBoxGLCM_Dissimilarity.isSelected());
        Settings.setBoolean("LIFEx.texture.GLCM.isEnergy", jCheckBoxGLCM_Energy.isSelected());
        Settings.setBoolean("LIFEx.texture.GLCM.isEntropy", jCheckBoxGLCM_Entropy.isSelected());
        Settings.setBoolean("LIFEx.texture.GLCM.isHomogeneity", jCheckBoxGLCM_Homogeneity.isSelected());
        Settings.setBoolean("LIFEx.texture.GLCM.isSave", jCheckBoxGLCM_Save.isSelected());
        Settings.setInt("LIFEx.texture.GLCM.distanceWithNeighbours", jComboBoxDistance.getSelectedIndex()+1); // be carreful index0 = 1vx
        
        Settings.setBoolean("LIFEx.texture.GLRLM.isGLNU", jCheckBoxGLRLM_GLNU.isSelected());
        Settings.setBoolean("LIFEx.texture.GLRLM.isHGRE", jCheckBoxGLRLM_HGRE.isSelected());
        Settings.setBoolean("LIFEx.texture.GLRLM.isLGRE", jCheckBoxGLRLM_LGRE.isSelected());
        Settings.setBoolean("LIFEx.texture.GLRLM.isLRE", jCheckBoxGLRLM_LRE.isSelected());
        Settings.setBoolean("LIFEx.texture.GLRLM.isLRHGE", jCheckBoxGLRLM_LRHGE.isSelected());
        Settings.setBoolean("LIFEx.texture.GLRLM.isLRLGE", jCheckBoxGLRLM_LRLGE.isSelected());
        Settings.setBoolean("LIFEx.texture.GLRLM.isRLNU", jCheckBoxGLRLM_RLNU.isSelected());
        Settings.setBoolean("LIFEx.texture.GLRLM.isRP", jCheckBoxGLRLM_RP.isSelected());
        Settings.setBoolean("LIFEx.texture.GLRLM.isSRE", jCheckBoxGLRLM_SRE.isSelected());
        Settings.setBoolean("LIFEx.texture.GLRLM.isSRHGE", jCheckBoxGLRLM_SRHGE.isSelected());
        Settings.setBoolean("LIFEx.texture.GLRLM.isSRLG", jCheckBoxGLRLM_SRLGE.isSelected());
        Settings.setBoolean("LIFEx.texture.GLRLM.isSave", jCheckBoxGLRLM_Save.isSelected());
        
        Settings.setBoolean("LIFEx.texture.GLZLM.isGLNU", jCheckBoxGLZLM_GLNU.isSelected());
        Settings.setBoolean("LIFEx.texture.GLZLM.isHGZE", jCheckBoxGLZLM_HGZE.isSelected());
        Settings.setBoolean("LIFEx.texture.GLZLM.isLGZE", jCheckBoxGLZLM_LGZE.isSelected());
        Settings.setBoolean("LIFEx.texture.GLZLM.isLZE", jCheckBoxGLZLM_LZE.isSelected());
        Settings.setBoolean("LIFEx.texture.GLZLM.isLZHGE", jCheckBoxGLZLM_LZHGE.isSelected());
        Settings.setBoolean("LIFEx.texture.GLZLM.isLZLGE", jCheckBoxGLZLM_LZLGE.isSelected());
        Settings.setBoolean("LIFEx.texture.GLZLM.isSZE", jCheckBoxGLZLM_SZE.isSelected());
        Settings.setBoolean("LIFEx.texture.GLZLM.isSZHGE", jCheckBoxGLZLM_SZHGE.isSelected());
        Settings.setBoolean("LIFEx.texture.GLZLM.isSZLGE", jCheckBoxGLZLM_SZLGE.isSelected());
        Settings.setBoolean("LIFEx.texture.GLZLM.isSave", jCheckBoxGLZLM_Save.isSelected());
        Settings.setBoolean("LIFEx.texture.GLZLM.isZLNU", jCheckBoxGLZLM_ZLNU.isSelected());
        Settings.setBoolean("LIFEx.texture.GLZLM.isZP", jCheckBoxGLZLM_ZP.isSelected());
        
        Settings.setBoolean("LIFEx.texture.NGLDM.isBusyness", jCheckBoxNGLDM_Busyness.isSelected());
        Settings.setBoolean("LIFEx.texture.NGLDM.isCoarseness", jCheckBoxNGLDM_Coarseness.isSelected());
        Settings.setBoolean("LIFEx.texture.NGLDM.isContrast", jCheckBoxNGLDM_Contrast.isSelected());
        Settings.setBoolean("LIFEx.texture.NGLDM.isSave", jCheckBoxNGLDM_Save.isSelected());
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanelFirstOrder = new javax.swing.JPanel();
        jLabelRotateFirstOrder = new bardp.utility.swing.JLabelRotate();
        jLabel2 = new javax.swing.JLabel();
        jPanel19 = new javax.swing.JPanel();
        jPanelShape = new javax.swing.JPanel();
        jCheckBoxShape_Sphericity = new javax.swing.JCheckBox();
        jCheckBoxShape_Compacity = new javax.swing.JCheckBox();
        jCheckBoxShape_Volume_mL = new javax.swing.JCheckBox();
        jCheckBoxShape_Volume_vx = new javax.swing.JCheckBox();
        jPanelHistogram = new javax.swing.JPanel();
        jCheckBoxHistogram_Skewness = new javax.swing.JCheckBox();
        jCheckBoxHistogram_Kurtosis = new javax.swing.JCheckBox();
        jCheckBoxHistogram_Entropy_log10 = new javax.swing.JCheckBox();
        jCheckBoxHistogram_Entropy_log2 = new javax.swing.JCheckBox();
        jCheckBoxHistogram_Energy = new javax.swing.JCheckBox();
        jLabel22 = new javax.swing.JLabel();
        jLabel23 = new javax.swing.JLabel();
        jPanelConventional = new javax.swing.JPanel();
        jCheckBoxConventional_SUVmin_mean_max = new javax.swing.JCheckBox();
        jCheckBoxConventional_SUVpeakSphere05mL = new javax.swing.JCheckBox();
        jCheckBoxConventional_SUVpeakSphere1mL = new javax.swing.JCheckBox();
        jCheckBoxConventional_CalciumAgatstonScore = new javax.swing.JCheckBox();
        jCheckBoxConventional_TLGmL = new javax.swing.JCheckBox();
        jLabel26 = new javax.swing.JLabel();
        jCheckBoxConventional_RIMmin_mean_max = new javax.swing.JCheckBox();
        jCheckBoxConventional_Q1_Q2_Q3 = new javax.swing.JCheckBox();
        jPanelSecondOrder = new javax.swing.JPanel();
        jLabelRotateSecondOrder = new bardp.utility.swing.JLabelRotate();
        jLabel1 = new javax.swing.JLabel();
        jPanel17 = new javax.swing.JPanel();
        jPanelGLCM = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        jComboBoxDistance = new javax.swing.JComboBox<>();
        jPanel2 = new javax.swing.JPanel();
        jLabelDistance = new javax.swing.JLabel();
        jLabelDistance1 = new javax.swing.JLabel();
        jPanel11 = new javax.swing.JPanel();
        jCheckBoxGLCM_Homogeneity = new javax.swing.JCheckBox();
        jPanel12 = new javax.swing.JPanel();
        jCheckBoxGLCM_Energy = new javax.swing.JCheckBox();
        jPanel13 = new javax.swing.JPanel();
        jCheckBoxGLCM_Contrast = new javax.swing.JCheckBox();
        jPanel14 = new javax.swing.JPanel();
        jCheckBoxGLCM_Correlation = new javax.swing.JCheckBox();
        jPanel15 = new javax.swing.JPanel();
        jCheckBoxGLCM_Entropy = new javax.swing.JCheckBox();
        jPanel16 = new javax.swing.JPanel();
        jCheckBoxGLCM_Dissimilarity = new javax.swing.JCheckBox();
        jPanelGLRM = new javax.swing.JPanel();
        jCheckBoxGLRLM_SRE = new javax.swing.JCheckBox();
        jCheckBoxGLRLM_LRE = new javax.swing.JCheckBox();
        jCheckBoxGLRLM_LGRE = new javax.swing.JCheckBox();
        jCheckBoxGLRLM_HGRE = new javax.swing.JCheckBox();
        jCheckBoxGLRLM_SRLGE = new javax.swing.JCheckBox();
        jCheckBoxGLRLM_SRHGE = new javax.swing.JCheckBox();
        jCheckBoxGLRLM_LRLGE = new javax.swing.JCheckBox();
        jCheckBoxGLRLM_LRHGE = new javax.swing.JCheckBox();
        jCheckBoxGLRLM_GLNU = new javax.swing.JCheckBox();
        jCheckBoxGLRLM_RLNU = new javax.swing.JCheckBox();
        jCheckBoxGLRLM_RP = new javax.swing.JCheckBox();
        jPanelNGLDM = new javax.swing.JPanel();
        jCheckBoxNGLDM_Coarseness = new javax.swing.JCheckBox();
        jCheckBoxNGLDM_Contrast = new javax.swing.JCheckBox();
        jCheckBoxNGLDM_Busyness = new javax.swing.JCheckBox();
        jPanelGLZLM = new javax.swing.JPanel();
        jCheckBoxGLZLM_SZE = new javax.swing.JCheckBox();
        jCheckBoxGLZLM_LZE = new javax.swing.JCheckBox();
        jCheckBoxGLZLM_LGZE = new javax.swing.JCheckBox();
        jCheckBoxGLZLM_HGZE = new javax.swing.JCheckBox();
        jCheckBoxGLZLM_SZLGE = new javax.swing.JCheckBox();
        jCheckBoxGLZLM_SZHGE = new javax.swing.JCheckBox();
        jCheckBoxGLZLM_LZLGE = new javax.swing.JCheckBox();
        jCheckBoxGLZLM_LZHGE = new javax.swing.JCheckBox();
        jCheckBoxGLZLM_GLNU = new javax.swing.JCheckBox();
        jCheckBoxGLZLM_ZLNU = new javax.swing.JCheckBox();
        jCheckBoxGLZLM_ZP = new javax.swing.JCheckBox();
        jPanelResults = new javax.swing.JPanel();
        jLabelRotateResults = new bardp.utility.swing.JLabelRotate();
        jLabel3 = new javax.swing.JLabel();
        jPanel10 = new javax.swing.JPanel();
        jPanelHistogramFile = new javax.swing.JPanel();
        jPanel20 = new javax.swing.JPanel();
        jCheckBoxHISTO_Show = new javax.swing.JCheckBox();
        jCheckBoxHISTO_Save = new javax.swing.JCheckBox();
        jPanelMatrixFile = new javax.swing.JPanel();
        jPanel18 = new javax.swing.JPanel();
        jPanelGLCM_Save = new javax.swing.JPanel();
        jCheckBoxGLCM_Save = new javax.swing.JCheckBox();
        jLabelInEachDirection = new javax.swing.JLabel();
        jCheckBoxGLRLM_Save = new javax.swing.JCheckBox();
        jCheckBoxNGLDM_Save = new javax.swing.JCheckBox();
        jCheckBoxGLZLM_Save = new javax.swing.JCheckBox();

        setLayout(new javax.swing.BoxLayout(this, javax.swing.BoxLayout.Y_AXIS));

        jPanelFirstOrder.setLayout(new javax.swing.BoxLayout(jPanelFirstOrder, javax.swing.BoxLayout.LINE_AXIS));

        jLabelRotateFirstOrder.setText("First Order");
        jLabelRotateFirstOrder.setAlignmentY(0.8F);
        jPanelFirstOrder.add(jLabelRotateFirstOrder);

        jLabel2.setText("     ");
        jPanelFirstOrder.add(jLabel2);

        jPanel19.setOpaque(false);
        java.awt.FlowLayout flowLayout3 = new java.awt.FlowLayout(java.awt.FlowLayout.LEFT);
        flowLayout3.setAlignOnBaseline(true);
        jPanel19.setLayout(flowLayout3);

        jPanelShape.setOpaque(false);
        jPanelShape.setLayout(new javax.swing.BoxLayout(jPanelShape, javax.swing.BoxLayout.Y_AXIS));

        jCheckBoxShape_Sphericity.setText("SHAPE_Sphericity");
        jCheckBoxShape_Sphericity.setContentAreaFilled(false);
        jPanelShape.add(jCheckBoxShape_Sphericity);

        jCheckBoxShape_Compacity.setText("SHAPE_Compacity");
        jCheckBoxShape_Compacity.setContentAreaFilled(false);
        jPanelShape.add(jCheckBoxShape_Compacity);

        jCheckBoxShape_Volume_mL.setText("SHAPE_Volume_mL");
        jCheckBoxShape_Volume_mL.setContentAreaFilled(false);
        jPanelShape.add(jCheckBoxShape_Volume_mL);

        jCheckBoxShape_Volume_vx.setText("SHAPE_Volume_vx");
        jCheckBoxShape_Volume_vx.setContentAreaFilled(false);
        jPanelShape.add(jCheckBoxShape_Volume_vx);

        jPanel19.add(jPanelShape);

        jPanelHistogram.setOpaque(false);
        jPanelHistogram.setLayout(new javax.swing.BoxLayout(jPanelHistogram, javax.swing.BoxLayout.Y_AXIS));

        jCheckBoxHistogram_Skewness.setText("HISTO_Skewness");
        jCheckBoxHistogram_Skewness.setContentAreaFilled(false);
        jPanelHistogram.add(jCheckBoxHistogram_Skewness);

        jCheckBoxHistogram_Kurtosis.setText("HISTO_Kurtosis");
        jCheckBoxHistogram_Kurtosis.setContentAreaFilled(false);
        jPanelHistogram.add(jCheckBoxHistogram_Kurtosis);

        jCheckBoxHistogram_Entropy_log10.setText("HISTO_Entropy_log10");
        jCheckBoxHistogram_Entropy_log10.setContentAreaFilled(false);
        jPanelHistogram.add(jCheckBoxHistogram_Entropy_log10);

        jCheckBoxHistogram_Entropy_log2.setText("HISTO_Entropy_log2");
        jCheckBoxHistogram_Entropy_log2.setContentAreaFilled(false);
        jPanelHistogram.add(jCheckBoxHistogram_Entropy_log2);

        jCheckBoxHistogram_Energy.setText("HISTO_Energy");
        jCheckBoxHistogram_Energy.setContentAreaFilled(false);
        jPanelHistogram.add(jCheckBoxHistogram_Energy);
        jPanelHistogram.add(jLabel22);
        jPanelHistogram.add(jLabel23);

        jPanel19.add(jPanelHistogram);

        jPanelConventional.setOpaque(false);
        jPanelConventional.setLayout(new javax.swing.BoxLayout(jPanelConventional, javax.swing.BoxLayout.Y_AXIS));

        jCheckBoxConventional_SUVmin_mean_max.setText("CONV/DISCRET_(min, mean, sd, max)");
        jCheckBoxConventional_SUVmin_mean_max.setContentAreaFilled(false);
        jPanelConventional.add(jCheckBoxConventional_SUVmin_mean_max);

        jCheckBoxConventional_SUVpeakSphere05mL.setText("CONV/DISCRET_peak 0.5mL");
        jCheckBoxConventional_SUVpeakSphere05mL.setContentAreaFilled(false);
        jPanelConventional.add(jCheckBoxConventional_SUVpeakSphere05mL);

        jCheckBoxConventional_SUVpeakSphere1mL.setText("CONV/DISCRET_peak 1mL");
        jCheckBoxConventional_SUVpeakSphere1mL.setContentAreaFilled(false);
        jPanelConventional.add(jCheckBoxConventional_SUVpeakSphere1mL);

        jCheckBoxConventional_CalciumAgatstonScore.setText("CONV/DISCRET_CAS");
        jCheckBoxConventional_CalciumAgatstonScore.setToolTipText("Calcium Agatston Score");
        jCheckBoxConventional_CalciumAgatstonScore.setContentAreaFilled(false);
        jPanelConventional.add(jCheckBoxConventional_CalciumAgatstonScore);

        jCheckBoxConventional_TLGmL.setText("CONV/DISCRET_TLG (for TP, MN)");
        jCheckBoxConventional_TLGmL.setContentAreaFilled(false);
        jPanelConventional.add(jCheckBoxConventional_TLGmL);
        jPanelConventional.add(jLabel26);

        jCheckBoxConventional_RIMmin_mean_max.setText("CONV/DISCRET_RIM_(min, mean, sd, max, ...)");
        jCheckBoxConventional_RIMmin_mean_max.setToolTipText("Envelope Intensity of successive layers of voxels from the outside of the region to the inside.");
        jCheckBoxConventional_RIMmin_mean_max.setContentAreaFilled(false);
        jPanelConventional.add(jCheckBoxConventional_RIMmin_mean_max);

        jCheckBoxConventional_Q1_Q2_Q3.setText("CONV/DISCRET_(Q1, Q2, Q3)");
        jCheckBoxConventional_Q1_Q2_Q3.setContentAreaFilled(false);
        jPanelConventional.add(jCheckBoxConventional_Q1_Q2_Q3);

        jPanel19.add(jPanelConventional);

        jPanelFirstOrder.add(jPanel19);

        add(jPanelFirstOrder);

        jPanelSecondOrder.setLayout(new javax.swing.BoxLayout(jPanelSecondOrder, javax.swing.BoxLayout.LINE_AXIS));

        jLabelRotateSecondOrder.setText("Second Order");
        jLabelRotateSecondOrder.setAlignmentY(0.8F);
        jPanelSecondOrder.add(jLabelRotateSecondOrder);

        jLabel1.setText("     ");
        jPanelSecondOrder.add(jLabel1);

        jPanel17.setOpaque(false);
        java.awt.FlowLayout flowLayout2 = new java.awt.FlowLayout(java.awt.FlowLayout.LEFT);
        flowLayout2.setAlignOnBaseline(true);
        jPanel17.setLayout(flowLayout2);

        jPanelGLCM.setOpaque(false);
        jPanelGLCM.setLayout(new javax.swing.BoxLayout(jPanelGLCM, javax.swing.BoxLayout.Y_AXIS));

        jPanel1.setOpaque(false);

        jComboBoxDistance.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "1", "2", "3", "4", "5", "6", "7", "8", "9", "10" }));
        jComboBoxDistance.setOpaque(false);
        jComboBoxDistance.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBoxDistanceActionPerformed(evt);
            }
        });
        jPanel1.add(jComboBoxDistance);

        jPanel2.setOpaque(false);
        jPanel2.setLayout(new javax.swing.BoxLayout(jPanel2, javax.swing.BoxLayout.Y_AXIS));

        jLabelDistance.setText("vx : distance");
        jPanel2.add(jLabelDistance);

        jLabelDistance1.setText("with neighbours");
        jPanel2.add(jLabelDistance1);

        jPanel1.add(jPanel2);

        jPanelGLCM.add(jPanel1);

        jPanel11.setOpaque(false);
        jPanel11.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER, 5, 0));

        jCheckBoxGLCM_Homogeneity.setText("GLCM_Homogeneity");
        jCheckBoxGLCM_Homogeneity.setContentAreaFilled(false);
        jCheckBoxGLCM_Homogeneity.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        jPanel11.add(jCheckBoxGLCM_Homogeneity);

        jPanelGLCM.add(jPanel11);

        jPanel12.setOpaque(false);
        jPanel12.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 5, 0));

        jCheckBoxGLCM_Energy.setText("GLCM_Energy");
        jCheckBoxGLCM_Energy.setContentAreaFilled(false);
        jPanel12.add(jCheckBoxGLCM_Energy);

        jPanelGLCM.add(jPanel12);

        jPanel13.setOpaque(false);
        jPanel13.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 5, 0));

        jCheckBoxGLCM_Contrast.setText("GLCM_Contrast");
        jCheckBoxGLCM_Contrast.setContentAreaFilled(false);
        jPanel13.add(jCheckBoxGLCM_Contrast);

        jPanelGLCM.add(jPanel13);

        jPanel14.setOpaque(false);
        jPanel14.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 5, 0));

        jCheckBoxGLCM_Correlation.setText("GLCM_Correlation");
        jCheckBoxGLCM_Correlation.setContentAreaFilled(false);
        jPanel14.add(jCheckBoxGLCM_Correlation);

        jPanelGLCM.add(jPanel14);

        jPanel15.setOpaque(false);
        jPanel15.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 5, 0));

        jCheckBoxGLCM_Entropy.setText("GLCM_Entropy");
        jCheckBoxGLCM_Entropy.setContentAreaFilled(false);
        jPanel15.add(jCheckBoxGLCM_Entropy);

        jPanelGLCM.add(jPanel15);

        jPanel16.setOpaque(false);
        jPanel16.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 5, 0));

        jCheckBoxGLCM_Dissimilarity.setText("GLCM_Dissimilarity");
        jCheckBoxGLCM_Dissimilarity.setContentAreaFilled(false);
        jPanel16.add(jCheckBoxGLCM_Dissimilarity);

        jPanelGLCM.add(jPanel16);

        jPanel17.add(jPanelGLCM);

        jPanelGLRM.setOpaque(false);
        jPanelGLRM.setLayout(new javax.swing.BoxLayout(jPanelGLRM, javax.swing.BoxLayout.Y_AXIS));

        jCheckBoxGLRLM_SRE.setText("GLRM_SRE");
        jCheckBoxGLRLM_SRE.setContentAreaFilled(false);
        jPanelGLRM.add(jCheckBoxGLRLM_SRE);

        jCheckBoxGLRLM_LRE.setText("GLRM_LRE");
        jCheckBoxGLRLM_LRE.setContentAreaFilled(false);
        jPanelGLRM.add(jCheckBoxGLRLM_LRE);

        jCheckBoxGLRLM_LGRE.setText("GLRM_LGRE");
        jCheckBoxGLRLM_LGRE.setContentAreaFilled(false);
        jPanelGLRM.add(jCheckBoxGLRLM_LGRE);

        jCheckBoxGLRLM_HGRE.setText("GLRM_HGRE");
        jCheckBoxGLRLM_HGRE.setContentAreaFilled(false);
        jPanelGLRM.add(jCheckBoxGLRLM_HGRE);

        jCheckBoxGLRLM_SRLGE.setText("GLRM_SRLGE");
        jCheckBoxGLRLM_SRLGE.setContentAreaFilled(false);
        jPanelGLRM.add(jCheckBoxGLRLM_SRLGE);

        jCheckBoxGLRLM_SRHGE.setText("GLRM_SRHGE");
        jCheckBoxGLRLM_SRHGE.setContentAreaFilled(false);
        jPanelGLRM.add(jCheckBoxGLRLM_SRHGE);

        jCheckBoxGLRLM_LRLGE.setText("GLRM_LRLGE");
        jCheckBoxGLRLM_LRLGE.setContentAreaFilled(false);
        jPanelGLRM.add(jCheckBoxGLRLM_LRLGE);

        jCheckBoxGLRLM_LRHGE.setText("GLRM_LRHGE");
        jCheckBoxGLRLM_LRHGE.setContentAreaFilled(false);
        jPanelGLRM.add(jCheckBoxGLRLM_LRHGE);

        jCheckBoxGLRLM_GLNU.setText("GLRM_GLNU");
        jCheckBoxGLRLM_GLNU.setContentAreaFilled(false);
        jPanelGLRM.add(jCheckBoxGLRLM_GLNU);

        jCheckBoxGLRLM_RLNU.setText("GLRM_RLNU");
        jCheckBoxGLRLM_RLNU.setContentAreaFilled(false);
        jPanelGLRM.add(jCheckBoxGLRLM_RLNU);

        jCheckBoxGLRLM_RP.setText("GLRM_RP");
        jCheckBoxGLRLM_RP.setContentAreaFilled(false);
        jPanelGLRM.add(jCheckBoxGLRLM_RP);

        jPanel17.add(jPanelGLRM);

        jPanelNGLDM.setOpaque(false);
        jPanelNGLDM.setLayout(new javax.swing.BoxLayout(jPanelNGLDM, javax.swing.BoxLayout.Y_AXIS));

        jCheckBoxNGLDM_Coarseness.setText("NGLDM_Coarseness");
        jCheckBoxNGLDM_Coarseness.setContentAreaFilled(false);
        jPanelNGLDM.add(jCheckBoxNGLDM_Coarseness);

        jCheckBoxNGLDM_Contrast.setText("NGLDM_Contrast");
        jCheckBoxNGLDM_Contrast.setContentAreaFilled(false);
        jPanelNGLDM.add(jCheckBoxNGLDM_Contrast);

        jCheckBoxNGLDM_Busyness.setText("NGLDM_Busyness");
        jCheckBoxNGLDM_Busyness.setContentAreaFilled(false);
        jPanelNGLDM.add(jCheckBoxNGLDM_Busyness);

        jPanel17.add(jPanelNGLDM);

        jPanelGLZLM.setOpaque(false);
        jPanelGLZLM.setLayout(new javax.swing.BoxLayout(jPanelGLZLM, javax.swing.BoxLayout.Y_AXIS));

        jCheckBoxGLZLM_SZE.setText("GLZLM_SZE");
        jCheckBoxGLZLM_SZE.setContentAreaFilled(false);
        jPanelGLZLM.add(jCheckBoxGLZLM_SZE);

        jCheckBoxGLZLM_LZE.setText("GLZLM_LZE");
        jCheckBoxGLZLM_LZE.setContentAreaFilled(false);
        jPanelGLZLM.add(jCheckBoxGLZLM_LZE);

        jCheckBoxGLZLM_LGZE.setText("GLZLM_LGZE");
        jCheckBoxGLZLM_LGZE.setContentAreaFilled(false);
        jPanelGLZLM.add(jCheckBoxGLZLM_LGZE);

        jCheckBoxGLZLM_HGZE.setText("GLZLM_HGZE");
        jCheckBoxGLZLM_HGZE.setContentAreaFilled(false);
        jPanelGLZLM.add(jCheckBoxGLZLM_HGZE);

        jCheckBoxGLZLM_SZLGE.setText("GLZLM_SZLGE");
        jCheckBoxGLZLM_SZLGE.setContentAreaFilled(false);
        jPanelGLZLM.add(jCheckBoxGLZLM_SZLGE);

        jCheckBoxGLZLM_SZHGE.setText("GLZLM_SZHGE");
        jCheckBoxGLZLM_SZHGE.setContentAreaFilled(false);
        jPanelGLZLM.add(jCheckBoxGLZLM_SZHGE);

        jCheckBoxGLZLM_LZLGE.setText("GLZLM_LZLGE");
        jCheckBoxGLZLM_LZLGE.setContentAreaFilled(false);
        jPanelGLZLM.add(jCheckBoxGLZLM_LZLGE);

        jCheckBoxGLZLM_LZHGE.setText("GLZLM_LZHGE");
        jCheckBoxGLZLM_LZHGE.setContentAreaFilled(false);
        jPanelGLZLM.add(jCheckBoxGLZLM_LZHGE);

        jCheckBoxGLZLM_GLNU.setText("GLZLM_GLNU");
        jCheckBoxGLZLM_GLNU.setContentAreaFilled(false);
        jPanelGLZLM.add(jCheckBoxGLZLM_GLNU);

        jCheckBoxGLZLM_ZLNU.setText("GLZLM_ZLNU");
        jCheckBoxGLZLM_ZLNU.setContentAreaFilled(false);
        jPanelGLZLM.add(jCheckBoxGLZLM_ZLNU);

        jCheckBoxGLZLM_ZP.setText("GLZLM_ZP");
        jCheckBoxGLZLM_ZP.setContentAreaFilled(false);
        jPanelGLZLM.add(jCheckBoxGLZLM_ZP);

        jPanel17.add(jPanelGLZLM);

        jPanelSecondOrder.add(jPanel17);

        add(jPanelSecondOrder);

        jPanelResults.setLayout(new javax.swing.BoxLayout(jPanelResults, javax.swing.BoxLayout.LINE_AXIS));

        jLabelRotateResults.setText("Output");
        jLabelRotateResults.setAlignmentY(0.8F);
        jPanelResults.add(jLabelRotateResults);

        jLabel3.setText("     ");
        jPanelResults.add(jLabel3);

        jPanel10.setOpaque(false);
        java.awt.FlowLayout flowLayout1 = new java.awt.FlowLayout(java.awt.FlowLayout.LEFT);
        flowLayout1.setAlignOnBaseline(true);
        jPanel10.setLayout(flowLayout1);

        jPanelHistogramFile.setOpaque(false);
        jPanelHistogramFile.setLayout(new javax.swing.BoxLayout(jPanelHistogramFile, javax.swing.BoxLayout.Y_AXIS));

        jPanel20.setOpaque(false);
        jPanel20.setLayout(new javax.swing.BoxLayout(jPanel20, javax.swing.BoxLayout.Y_AXIS));
        jPanelHistogramFile.add(jPanel20);

        jCheckBoxHISTO_Show.setText("Show histogram");
        jCheckBoxHISTO_Show.setContentAreaFilled(false);
        jPanelHistogramFile.add(jCheckBoxHISTO_Show);

        jCheckBoxHISTO_Save.setText("Save histogram in file");
        jCheckBoxHISTO_Save.setContentAreaFilled(false);
        jPanelHistogramFile.add(jCheckBoxHISTO_Save);

        jPanel10.add(jPanelHistogramFile);

        jPanelMatrixFile.setOpaque(false);
        jPanelMatrixFile.setLayout(new javax.swing.BoxLayout(jPanelMatrixFile, javax.swing.BoxLayout.Y_AXIS));

        jPanel18.setOpaque(false);
        jPanel18.setLayout(new javax.swing.BoxLayout(jPanel18, javax.swing.BoxLayout.Y_AXIS));

        jPanelGLCM_Save.setOpaque(false);
        jPanelGLCM_Save.setLayout(new javax.swing.BoxLayout(jPanelGLCM_Save, javax.swing.BoxLayout.Y_AXIS));

        jCheckBoxGLCM_Save.setBackground(bardp.ressources.Res.COLOR_BACKGROUND2);
        jCheckBoxGLCM_Save.setText("Save GLCM in file");
        jCheckBoxGLCM_Save.setContentAreaFilled(false);
        jPanelGLCM_Save.add(jCheckBoxGLCM_Save);

        jLabelInEachDirection.setText("    in each direction");
        jPanelGLCM_Save.add(jLabelInEachDirection);

        jPanel18.add(jPanelGLCM_Save);

        jPanelMatrixFile.add(jPanel18);

        jCheckBoxGLRLM_Save.setText("Save GLRLM in file");
        jCheckBoxGLRLM_Save.setContentAreaFilled(false);
        jPanelMatrixFile.add(jCheckBoxGLRLM_Save);

        jCheckBoxNGLDM_Save.setText("Save NGLDM in file");
        jCheckBoxNGLDM_Save.setContentAreaFilled(false);
        jPanelMatrixFile.add(jCheckBoxNGLDM_Save);

        jCheckBoxGLZLM_Save.setText("Save GLZLM in file");
        jCheckBoxGLZLM_Save.setContentAreaFilled(false);
        jPanelMatrixFile.add(jCheckBoxGLZLM_Save);

        jPanel10.add(jPanelMatrixFile);

        jPanelResults.add(jPanel10);

        add(jPanelResults);
    }// </editor-fold>//GEN-END:initComponents

   private void jComboBoxDistanceActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBoxDistanceActionPerformed
      // TODO add your handling code here:
   }//GEN-LAST:event_jComboBoxDistanceActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    public javax.swing.JCheckBox jCheckBoxConventional_CalciumAgatstonScore;
    public javax.swing.JCheckBox jCheckBoxConventional_Q1_Q2_Q3;
    public javax.swing.JCheckBox jCheckBoxConventional_RIMmin_mean_max;
    public javax.swing.JCheckBox jCheckBoxConventional_SUVmin_mean_max;
    public javax.swing.JCheckBox jCheckBoxConventional_SUVpeakSphere05mL;
    public javax.swing.JCheckBox jCheckBoxConventional_SUVpeakSphere1mL;
    public javax.swing.JCheckBox jCheckBoxConventional_TLGmL;
    public javax.swing.JCheckBox jCheckBoxGLCM_Contrast;
    public javax.swing.JCheckBox jCheckBoxGLCM_Correlation;
    public javax.swing.JCheckBox jCheckBoxGLCM_Dissimilarity;
    public javax.swing.JCheckBox jCheckBoxGLCM_Energy;
    public javax.swing.JCheckBox jCheckBoxGLCM_Entropy;
    public javax.swing.JCheckBox jCheckBoxGLCM_Homogeneity;
    public javax.swing.JCheckBox jCheckBoxGLCM_Save;
    public javax.swing.JCheckBox jCheckBoxGLRLM_GLNU;
    public javax.swing.JCheckBox jCheckBoxGLRLM_HGRE;
    public javax.swing.JCheckBox jCheckBoxGLRLM_LGRE;
    public javax.swing.JCheckBox jCheckBoxGLRLM_LRE;
    public javax.swing.JCheckBox jCheckBoxGLRLM_LRHGE;
    public javax.swing.JCheckBox jCheckBoxGLRLM_LRLGE;
    public javax.swing.JCheckBox jCheckBoxGLRLM_RLNU;
    public javax.swing.JCheckBox jCheckBoxGLRLM_RP;
    public javax.swing.JCheckBox jCheckBoxGLRLM_SRE;
    public javax.swing.JCheckBox jCheckBoxGLRLM_SRHGE;
    public javax.swing.JCheckBox jCheckBoxGLRLM_SRLGE;
    public javax.swing.JCheckBox jCheckBoxGLRLM_Save;
    public javax.swing.JCheckBox jCheckBoxGLZLM_GLNU;
    public javax.swing.JCheckBox jCheckBoxGLZLM_HGZE;
    public javax.swing.JCheckBox jCheckBoxGLZLM_LGZE;
    public javax.swing.JCheckBox jCheckBoxGLZLM_LZE;
    public javax.swing.JCheckBox jCheckBoxGLZLM_LZHGE;
    public javax.swing.JCheckBox jCheckBoxGLZLM_LZLGE;
    public javax.swing.JCheckBox jCheckBoxGLZLM_SZE;
    public javax.swing.JCheckBox jCheckBoxGLZLM_SZHGE;
    public javax.swing.JCheckBox jCheckBoxGLZLM_SZLGE;
    public javax.swing.JCheckBox jCheckBoxGLZLM_Save;
    public javax.swing.JCheckBox jCheckBoxGLZLM_ZLNU;
    public javax.swing.JCheckBox jCheckBoxGLZLM_ZP;
    public javax.swing.JCheckBox jCheckBoxHISTO_Save;
    public javax.swing.JCheckBox jCheckBoxHISTO_Show;
    public javax.swing.JCheckBox jCheckBoxHistogram_Energy;
    public javax.swing.JCheckBox jCheckBoxHistogram_Entropy_log10;
    public javax.swing.JCheckBox jCheckBoxHistogram_Entropy_log2;
    public javax.swing.JCheckBox jCheckBoxHistogram_Kurtosis;
    public javax.swing.JCheckBox jCheckBoxHistogram_Skewness;
    public javax.swing.JCheckBox jCheckBoxNGLDM_Busyness;
    public javax.swing.JCheckBox jCheckBoxNGLDM_Coarseness;
    public javax.swing.JCheckBox jCheckBoxNGLDM_Contrast;
    public javax.swing.JCheckBox jCheckBoxNGLDM_Save;
    public javax.swing.JCheckBox jCheckBoxShape_Compacity;
    public javax.swing.JCheckBox jCheckBoxShape_Sphericity;
    public javax.swing.JCheckBox jCheckBoxShape_Volume_mL;
    public javax.swing.JCheckBox jCheckBoxShape_Volume_vx;
    public javax.swing.JComboBox<String> jComboBoxDistance;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel22;
    private javax.swing.JLabel jLabel23;
    private javax.swing.JLabel jLabel26;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabelDistance;
    private javax.swing.JLabel jLabelDistance1;
    private javax.swing.JLabel jLabelInEachDirection;
    private bardp.utility.swing.JLabelRotate jLabelRotateFirstOrder;
    private bardp.utility.swing.JLabelRotate jLabelRotateResults;
    private bardp.utility.swing.JLabelRotate jLabelRotateSecondOrder;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel10;
    private javax.swing.JPanel jPanel11;
    private javax.swing.JPanel jPanel12;
    private javax.swing.JPanel jPanel13;
    private javax.swing.JPanel jPanel14;
    private javax.swing.JPanel jPanel15;
    private javax.swing.JPanel jPanel16;
    private javax.swing.JPanel jPanel17;
    private javax.swing.JPanel jPanel18;
    private javax.swing.JPanel jPanel19;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel20;
    private javax.swing.JPanel jPanelConventional;
    private javax.swing.JPanel jPanelFirstOrder;
    private javax.swing.JPanel jPanelGLCM;
    public javax.swing.JPanel jPanelGLCM_Save;
    private javax.swing.JPanel jPanelGLRM;
    private javax.swing.JPanel jPanelGLZLM;
    private javax.swing.JPanel jPanelHistogram;
    private javax.swing.JPanel jPanelHistogramFile;
    private javax.swing.JPanel jPanelMatrixFile;
    private javax.swing.JPanel jPanelNGLDM;
    private javax.swing.JPanel jPanelResults;
    private javax.swing.JPanel jPanelSecondOrder;
    private javax.swing.JPanel jPanelShape;
    // End of variables declaration//GEN-END:variables
}
