/*
 * USE OF THE LIFEx SOFTWARE IS SUBJECT TO THE FOLLOWING TERMS AND CONDITIONS. 
 * IF YOU DO NOT ACCEPT THESE TERMS AND CONDITIONS, DO NOT DOWNLOAD THE SOFTWARE. 
 * NON-PROFESSIONAL USERS ARE NOTIFIED THAT THE SOFTWARE AND THE RISKS INHERENT TO ITS USE,
 * MODIFICATION AND/OR DEVELOPMENT AND REPRODUCTION ARE NOT COVERED BY ANY WARRANTY ON THE PART OF THE CEA. 
 * IT IS HIGHLY RECOMMENDED THAT YOU ONLY LOAD AND/OR USE THE SOFTWARE IF YOU HAVE SPECIALIZED IN-DEPTH INFORMATION TECHNOLOGY KNOW-HOW.
 * THE COMMISSARIAT A L’ENERGIE ATOMIQUE ET AUX ENERGIES ALTERNATIVES (FRENCH ATOMIC ENERGY AGENCY), 
 * a public scientific, technical and industrial establishment, 
 * having its registered office at 25 rue Leblanc – bâtiment le Ponant D – 75015 Paris cedex, 
 * hereinafter referred to as "CEA", hereby licenses the right to use the LIFEx software as per the terms and conditions set out below.  
 */
package bardp.protocol.protocolTexture.textureGLCM;

import bardp.raw.RawTZYX;
import bardp.raw.RawSparseZYX;
import bardp.utility.Utils;

/**
 * <p>
 * Title: Local Image Feature Extraction (LIFEx)</p>
 * <p>
 * Description: </p>
 * <p>
 * Copyright: Copyright (c) 2016</p>
 * <p>
 * Company: Inserm/CNRS/CEA</p>
 * <p>
 * @author Christophe Nioche, Inserm at lifexsoft.org
 * @version 6.0
 */
public class TextureGLCMSubRoutine {

    public TextureGLCMSubRoutine(
            final RawSparseZYX maskOnSeriesRaw,
            final RawTZYX rawZYXDiscret,
            final int nbDirections,
            final int[][] directions,
            final int dist,
            final int nbLevel
    ) {
        this.maskOnSeriesRaw = maskOnSeriesRaw;
        this.rawZYXDiscret = rawZYXDiscret;
        this.nbDirections = nbDirections;
        this.directions = directions;
        this.dist = dist;
        this.nbGreyLevels = nbLevel;
    }

    private void showImgForDebug() {
        
        int greyValueRef, grayValueLocal;
        
        int z, y, x;
        int[] rawZYX = maskOnSeriesRaw.convZYXInDenseFormatShort();
        int nz = maskOnSeriesRaw.nz;
        int ny = maskOnSeriesRaw.ny;
        int nx = maskOnSeriesRaw.nx;
        int nyx = ny * nx;
        
        System.out.println("GLCM print()");
        for (z = 0; z <  nz; z++) {
            System.out.println("");
            System.out.println("");
            System.out.print("z=" + z);
            for (y = ny-1; y >= 0; y--) {
                System.out.println("");
                for (x = nx-1; x>=0 ; x--) {
                    if (rawZYX[z * nyx + y * nx + x] != 0) {
                        greyValueRef = Math.round(rawZYXDiscret.getTZYX(rawZYXDiscret.t, z, y, x));
                        System.out.print(greyValueRef + " ");
                    } else {
                        System.out.print("0 ");
                    }
                    
                }
            }
        }
    }
    
    private void showGLCM(
            final double[][][] GLCM
    ) {
        System.out.println("GLCM print()");
        
        for (int i=0 ; i<nbGreyLevels ; i++) {
            System.out.println("iLevel=" + i);
        
            for (int j=0 ; j<nbGreyLevels ; j++) {
                for (int iDirection=0 ; iDirection<nbDirections ; iDirection++) {
                    System.out.print(" " + Utils.d2s(GLCM[iDirection][i][j], 0));
                }
                System.out.println("");
            }
        }
    }

    public double[][][] getGLCMDirection() throws Exception {

//showImgForDebug(); // fo debug only
        
        double[][][] GLCM = new double[nbDirections][nbGreyLevels][nbGreyLevels];

        int greyValueRef, grayValueLocal;
        int dx, dy, dz;
        int sz, sy, sx;
        int iDirection;

        int iz, iy, ix;

        int nz = maskOnSeriesRaw.nz;
        int ny = maskOnSeriesRaw.ny;
        int nx = maskOnSeriesRaw.nx;

        for (iz = 0; iz < nz; iz++) {
            for (iy = 0; iy < ny; iy++) {
                for (ix = 0; ix < nx; ix++) {
                    
                    if (maskOnSeriesRaw.getZYX(iz, iy, ix)>0) {
                        greyValueRef = (int) rawZYXDiscret.getTZYX(rawZYXDiscret.t, iz, iy, ix);

                        for (iDirection=0 ; iDirection<nbDirections ; iDirection++) {

                            dz = directions[2][iDirection]; // z
                            dy = directions[1][iDirection]; // y
                            dx = directions[0][iDirection]; // x

                            dz *= dist;
                            dy *= dist;
                            dx *= dist;

                            sz = iz + dz;
                            sy = iy + dy;
                            sx = ix + dx;

                            try {
                                if (maskOnSeriesRaw.getZYX(sz, sy, sx)>0) {
                                    grayValueLocal = (int) rawZYXDiscret.getTZYX(rawZYXDiscret.t, sz, sy, sx);
                                    GLCM[iDirection][greyValueRef][grayValueLocal]++;
                                }
                            } catch (ArrayIndexOutOfBoundsException ignore) {
                            }
                        }
                    }
                }
            }
        }
        return GLCM;
    }
    
    RawSparseZYX maskOnSeriesRaw;
    RawTZYX rawZYXDiscret;

    /**
     * number of total directions: 13 for 3D, 4 for 2D
     */
    int nbDirections;

    /**
     * cf. TexGLCM for explanation
     */
    int[][] directions;

    /**
     * cf. TexGLCM for explanation
     */
    int dist;

    /**
     * number of grey level analysis
     */
    int nbGreyLevels;
}
