/*
 * USE OF THE LIFEx SOFTWARE IS SUBJECT TO THE FOLLOWING TERMS AND CONDITIONS. 
 * IF YOU DO NOT ACCEPT THESE TERMS AND CONDITIONS, DO NOT DOWNLOAD THE SOFTWARE. 
 * NON-PROFESSIONAL USERS ARE NOTIFIED THAT THE SOFTWARE AND THE RISKS INHERENT TO ITS USE,
 * MODIFICATION AND/OR DEVELOPMENT AND REPRODUCTION ARE NOT COVERED BY ANY WARRANTY ON THE PART OF THE CEA. 
 * IT IS HIGHLY RECOMMENDED THAT YOU ONLY LOAD AND/OR USE THE SOFTWARE IF YOU HAVE SPECIALIZED IN-DEPTH INFORMATION TECHNOLOGY KNOW-HOW.
 * THE COMMISSARIAT A L’ENERGIE ATOMIQUE ET AUX ENERGIES ALTERNATIVES (FRENCH ATOMIC ENERGY AGENCY), 
 * a public scientific, technical and industrial establishment, 
 * having its registered office at 25 rue Leblanc – bâtiment le Ponant D – 75015 Paris cedex, 
 * hereinafter referred to as "CEA", hereby licenses the right to use the LIFEx software as per the terms and conditions set out below.  
 */
package bardp.protocol.protocolTexture.textureGLRLM;

import bardp.img.coor.CoorRange;
import bardp.raw.RawTZYX;
import bardp.raw.RawSparseZYX;

/**
 * <p>
 * Title: Local Image Feature Extraction (LIFEx)</p>
 * <p>
 * Description: </p>
 * <p>
 * Copyright: Copyright (c) 2016</p>
 * <p>
 * Company: Inserm/CNRS/CEA</p>
 * <p>
 * @author Christophe Nioche, Inserm at lifexsoft.org
 * @version 6.0
 */
public class TextureGLRLMSubRoutine implements Runnable {

    public TextureGLRLMSubRoutine(
            final int[][][] GLRLM,
            final RawTZYX rawZYXDiscret,
            final RawSparseZYX mask,
            final CoorRange rangeBounds,
            final int iDirection
    ) {
        this.GLRLM = GLRLM;
        this.rawZYXDiscret = rawZYXDiscret;
        this.mask = mask;
        this.rangeBounds = rangeBounds;
        this.iDirection = iDirection;
    }

    /**
     * procedure of processing the grey level run length matrix (GLRLM) at
     * iDirection
     * <p>
     *
     */
    @Override
    public void run() {

        short zero = (short) 0;

        //<editor-fold defaultstate="collapsed" desc="DEBUG: display discrete image and continue image">
    /*     double value;
        System.out.println("nb vx = " + mask.getCoorList().size());
        System.out.println("rb = " + rb.toString());
        for (int z = rb.min.getFloorZ(); z <= rb.max.getFloorZ(); z++) {
            System.out.println("z=" + (z - rb.min.getFloorZ() + 1));
            for (int x = rb.min.getFloorX(); x <= rb.max.getFloorX(); x++) {
                for (int y = rb.min.getFloorY(); y <= rb.max.getFloorY(); y++) {
                        value = rawZYXDiscret.getTZYX(0, z, y, x);
                        //value = mask.getXYZ(x, y, z);
                        System.out.printf(" %.4f ", value);
                }
                System.out.println("");
            }
        }*/
        //</editor-fold>
        int[][] directions = rangeBounds.max.getZ() + 1 - rangeBounds.min.getZ() >= 2
                ? DIRECTIONS_3D
                : DIRECTIONS_2D;

        int greyLevel;
        int maskValue;
        int a, b, c, xx, yy, zz;
        int alignVoxels;
        RawTZYX copie = this.rawZYXDiscret.copyDataTZYX();

        int da = directions[0][iDirection];
        int db = directions[1][iDirection];
        int dc = directions[2][iDirection];

        int iz, iy, ix;
        
        //<editor-fold defaultstate="collapsed" desc="xrange">
        int[] xrange = new int[rangeBounds.max.getX() - rangeBounds.min.getX() + 1];
        if (da >= 0) {
            int i = 0;
            for (ix = rangeBounds.min.getX(); ix <= rangeBounds.max.getX(); ix++) {
                xrange[i++] = ix;
            }
        } else {
            int i = 0;
            for (ix = rangeBounds.max.getX(); ix >= rangeBounds.min.getX(); ix--) {
                xrange[i++] = ix;
            }
        }
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="yrange">
        int[] yrange = new int[rangeBounds.max.getY() - rangeBounds.min.getY() + 1];
        if (db >= 0) {
            int i = 0;
            for (iy = rangeBounds.min.getY(); iy <= rangeBounds.max.getY(); iy++) {
                yrange[i++] = iy;
            }
        } else {
            int i = 0;
            for (iy = rangeBounds.max.getY(); iy >= rangeBounds.min.getY(); iy--) {
                yrange[i++] = iy;
            }
        }
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="zrange">
        int[] zrange = new int[rangeBounds.max.getZ() - rangeBounds.min.getZ() + 1];
        if (dc >= 0) {
            int i = 0;
            for (iz = rangeBounds.min.getZ(); iz <= rangeBounds.max.getZ(); iz++) {
                zrange[i++] = iz;
            }
        } else {
            int i = 0;
            for (iz = rangeBounds.max.getZ(); iz >= rangeBounds.min.getZ(); iz--) {
                zrange[i++] = iz;
            }
        }
        //</editor-fold>

        int it = 0;
        for (int iix : xrange) {
            for (int iiy : yrange) {
                for (int iiz : zrange) {

                    greyLevel = (int) copie.getTZYX(it, iiz, iiy, iix);
                    maskValue = mask.getXYZ(iix, iiy, iiz);
                    copie.setTZYX(it, iiz, iiy, iix, zero);

                    if (maskValue > 0 && greyLevel > 0) {

                        alignVoxels = 1; // minimum size of run = 1 (unique voxel)

                        a = da;
                        b = db;
                        c = dc;

                        xx = iix + a;
                        yy = iiy + b;
                        zz = iiz + c;

                        // while neighbour has the same intensity, continue
                        while (greyLevel == copie.getTZYX(it, zz, yy, xx)
                                && mask.getXYZ(xx, yy, zz) > 0) {

                            alignVoxels++;

                            copie.setTZYX(it, zz, yy, xx, zero);

                            a = a + da; // [0]=x
                            b = b + db; // [1]=y
                            c = c + dc; // [2]=z

                            xx = iix + a;
                            yy = iiy + b;
                            zz = iiz + c;
                        }

                        // Classement of size of run in GLRLM="Voxel-alignment"
                        synchronized (GLRLM) {
                            GLRLM[greyLevel - 1][alignVoxels - 1][iDirection]++;
                        }
                    }
                }
            }
        }
    }

    private final int[][][] GLRLM;

    /**
     * discrete image
     */
    private final RawTZYX rawZYXDiscret;
    private final RawSparseZYX mask;

    /**
     * be careful: inversion of Y and X
     */
    private static final int[][] DIRECTIONS_3D = {
        // direc 1  2   3  4  5   6  7  8   9  10  11  12 13
                {1, 1, +1, 1, 1, +1, 0, 0, +0, -1, -1, -1, 0}, // Y
                {0, 0, +0, 1, 1, +1, 1, 1, +1, +1, +1, +1, 0}, // X
                {1, 0, -1, 1, 0, -1, 1, 0, -1, +1, +0, -1, 1}  // Z
    };

    /**
     * be careful: inversion of Y and X
     */
    private static final int[][] DIRECTIONS_2D = {
        {1, 1, 0, -1}, // Y
        {0, 1, 1, +1}, // X
        {0, 0, 0, +0}  // Z
    };

    private final CoorRange rangeBounds;

    private final int iDirection;
}
