/*
 * USE OF THE LIFEx SOFTWARE IS SUBJECT TO THE FOLLOWING TERMS AND CONDITIONS. 
 * IF YOU DO NOT ACCEPT THESE TERMS AND CONDITIONS, DO NOT DOWNLOAD THE SOFTWARE. 
 * NON-PROFESSIONAL USERS ARE NOTIFIED THAT THE SOFTWARE AND THE RISKS INHERENT TO ITS USE,
 * MODIFICATION AND/OR DEVELOPMENT AND REPRODUCTION ARE NOT COVERED BY ANY WARRANTY ON THE PART OF THE CEA. 
 * IT IS HIGHLY RECOMMENDED THAT YOU ONLY LOAD AND/OR USE THE SOFTWARE IF YOU HAVE SPECIALIZED IN-DEPTH INFORMATION TECHNOLOGY KNOW-HOW.
 * THE COMMISSARIAT A L’ENERGIE ATOMIQUE ET AUX ENERGIES ALTERNATIVES (FRENCH ATOMIC ENERGY AGENCY), 
 * a public scientific, technical and industrial establishment, 
 * having its registered office at 25 rue Leblanc – bâtiment le Ponant D – 75015 Paris cedex, 
 * hereinafter referred to as "CEA", hereby licenses the right to use the LIFEx software as per the terms and conditions set out below.  
 */
package bardp.protocol.protocolTexture.textureGLZLM;

import bardp.img.coor.CoorRange;
import bardp.raw.RawTZYX;
import bardp.raw.RawSparseZYX;
import bardp.utility.MessageGui;
import java.util.HashMap;
import java.util.concurrent.Callable;

/**
 * <p>
 * Title: Local Image Feature Extraction (LIFEx)</p>
 * <p>
 * Description: </p>
 * <p>
 * Copyright: Copyright (c) 2016</p>
 * <p>
 * Company: Inserm/CNRS/CEA</p>
 * <p>
 * @author Christophe Nioche, Inserm at lifexsoft.org
 * @version 6.0
 */
public class TextureGLZLMSubRoutine implements Callable {

    public TextureGLZLMSubRoutine(
            final TextureGLZLMSparse texGLZLMSparse,
            final CoorRange rangeBounds,
            final RawTZYX rawZYXDiscret,
            final RawSparseZYX maskRaw,
            final int iGrayLevel,
            final MessageGui progressGUIGLZLM
    ) {
        this.texGLZLMSparse = texGLZLMSparse;
        this.rangeBounds = rangeBounds;
        this.rawZYXDiscret = rawZYXDiscret;
        this.maskRaw = maskRaw;
        this.iGrayLevel = iGrayLevel;
        this.messageGUI = progressGUIGLZLM;
    }

    @Override
    public Long call() throws Exception {

        teta = 0L;

        // define rawZYX from iGrayLevel researched
        RawSparseZYX maskiLevel = rawZYXDiscret.createMaskWithMaskAndValue(maskRaw, iGrayLevel);
        
        // make cluster
        TextureGLZLMCluster3D texCluster3D = new TextureGLZLMCluster3D(maskiLevel, rangeBounds);
        int clusterIdMax = texCluster3D.run();
        HashMap<Short, Integer> nbVoxels = texCluster3D.getNbVoxels();

        // be careful number 2 is the first id of first cluster
        int count;
        for (short id = 2; id <= clusterIdMax; id++) {
            count = nbVoxels.containsKey(id) ? nbVoxels.get(id) : 0;
            synchronized(texGLZLMSparse) {
                texGLZLMSparse.setInc(iGrayLevel, count);
            }
            if (count > 0) {
                teta++;
            }
        }

        messageGUI.incProgress();
        return teta;
    }

    private Long teta;
    private final TextureGLZLMSparse texGLZLMSparse;
    private final CoorRange rangeBounds;
    private final RawTZYX rawZYXDiscret;
    private final RawSparseZYX maskRaw;
    private final int iGrayLevel;
    private final MessageGui messageGUI;
}
