/*
 * USE OF THE LIFEx SOFTWARE IS SUBJECT TO THE FOLLOWING TERMS AND CONDITIONS. 
 * IF YOU DO NOT ACCEPT THESE TERMS AND CONDITIONS, DO NOT DOWNLOAD THE SOFTWARE. 
 * NON-PROFESSIONAL USERS ARE NOTIFIED THAT THE SOFTWARE AND THE RISKS INHERENT TO ITS USE,
 * MODIFICATION AND/OR DEVELOPMENT AND REPRODUCTION ARE NOT COVERED BY ANY WARRANTY ON THE PART OF THE CEA. 
 * IT IS HIGHLY RECOMMENDED THAT YOU ONLY LOAD AND/OR USE THE SOFTWARE IF YOU HAVE SPECIALIZED IN-DEPTH INFORMATION TECHNOLOGY KNOW-HOW.
 * THE COMMISSARIAT A L’ENERGIE ATOMIQUE ET AUX ENERGIES ALTERNATIVES (FRENCH ATOMIC ENERGY AGENCY), 
 * a public scientific, technical and industrial establishment, 
 * having its registered office at 25 rue Leblanc – bâtiment le Ponant D – 75015 Paris cedex, 
 * hereinafter referred to as "CEA", hereby licenses the right to use the LIFEx software as per the terms and conditions set out below.  
 */
package bardp.protocol.protocolTexture.traditional;

import bardp.img.coor.Coor;
import bardp.img.coor.CoorList;
import bardp.raw.RawTZYX;

/**
 * <p>
 * Title: Local Image Feature Extraction (LIFEx)</p>
 * <p>
 * Description: </p>
 * <p>
 * Copyright: Copyright (c) 2016</p>
 * <p>
 * Company: Inserm/CNRS/CEA</p>
 * <p>
 * @author Christophe Nioche, Inserm at lifexsoft.org
 * @version 6.0
 */
public class ConventionalRimExtractRawZYX {

    public ConventionalRimExtractRawZYX(
            final RawTZYX rawZYX,
            final CoorList coorList
    ) {
        this.rawZYX = rawZYX;
        this.coorList = coorList;
    }

    protected ConventionalRimModel run() {
        ConventionalRimModel rimResults = new ConventionalRimModel();

        rimResults.min = Double.MAX_VALUE;
        rimResults.max = -Double.MAX_VALUE;
        rimResults.n = 0;
        rimResults.mean = 0f;
        rimResults.sum = 0;
        rimResults.stdev = 0;
        double m2 = 0f;
        double delta;
        double value;

        if (!coorList.isEmpty()) {

            for (Coor coor : coorList) {

                // value
                value = rawZYX.getTZYX(rawZYX.t, coor);
                   
                // stdev...
                rimResults.sum += value;
                rimResults.n++;
                delta = value - rimResults.mean;
                rimResults.mean += delta / rimResults.n;
                m2 += delta * (value - rimResults.mean);

                // keep value min
                if (value < rimResults.min) {
                    rimResults.min = value;
                }

                // keep value max
                if (value > rimResults.max) {
                    rimResults.max = value;
                }
            }

            rimResults.cm3 = 
                    (rimResults.n 
                    * rawZYX.iopIppSp.spModel.getVoxelSpacing()
                    / 1_000d);
                    
            rimResults.stdev = Math.sqrt(m2 / rimResults.n);

            if (rimResults.min == Double.MAX_VALUE) {
                rimResults.min = 0f;
            }
            if (rimResults.max == -Double.MAX_VALUE) {
                rimResults.max = 0f;
            }

        }

        return rimResults;
    }

    final RawTZYX rawZYX;
    CoorList coorList;
}
