/*
 * USE OF THE LIFEx SOFTWARE IS SUBJECT TO THE FOLLOWING TERMS AND CONDITIONS. 
 * IF YOU DO NOT ACCEPT THESE TERMS AND CONDITIONS, DO NOT DOWNLOAD THE SOFTWARE. 
 * NON-PROFESSIONAL USERS ARE NOTIFIED THAT THE SOFTWARE AND THE RISKS INHERENT TO ITS USE,
 * MODIFICATION AND/OR DEVELOPMENT AND REPRODUCTION ARE NOT COVERED BY ANY WARRANTY ON THE PART OF THE CEA. 
 * IT IS HIGHLY RECOMMENDED THAT YOU ONLY LOAD AND/OR USE THE SOFTWARE IF YOU HAVE SPECIALIZED IN-DEPTH INFORMATION TECHNOLOGY KNOW-HOW.
 * THE COMMISSARIAT A L’ENERGIE ATOMIQUE ET AUX ENERGIES ALTERNATIVES (FRENCH ATOMIC ENERGY AGENCY), 
 * a public scientific, technical and industrial establishment, 
 * having its registered office at 25 rue Leblanc – bâtiment le Ponant D – 75015 Paris cedex, 
 * hereinafter referred to as "CEA", hereby licenses the right to use the LIFEx software as per the terms and conditions set out below.  
 */
package bardp.protocol.protocolTexture.traditional;

import bardp.gui.FrameGui;
import bardp.raw.RawSparseZYX;
import bardp.raw.RawTZYX;
import bardp.session.patient.study.series.SeriesModel;
import bardp.session.patient.study.series.SeriesUnits;
import bardp.session.patient.study.series.load.loadDcm.PT_NM.PT_NM;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.logging.Level;

/**
 * <p>
 * Title: Local Image Feature Extraction (LIFEx)</p>
 * <p>
 * Description: </p>
 * <p>
 * Copyright: Copyright (c) 2016</p>
 * <p>
 * Company: Inserm/CNRS/CEA</p>
 * <p>
 * @author Christophe Nioche, Inserm at lifexsoft.org
 * @version 6.0
 */
public class ConvertionDoubleUnitYOfNativeSeries {

    /**
     * unitY conversion if necessary (PT, NM modality) of rawZYX
     * <br>else return rawZYX directly (no conversion)
     * <br> return only values on CoorList
     *
     * @param series
     * @param rawZYXFloat
     * @param rawZYXSparse
     * @return
     */
    public double[] run(
            final SeriesModel series,
            final RawTZYX rawZYXFloat,
            final RawSparseZYX rawZYXSparse
    ) {

        //<editor-fold defaultstate="collapsed" desc="checking CoorList">
        if (rawZYXSparse == null) {
            FrameGui.LOGGER.log(
                    Level.SEVERE,
                    "ConvContinueSeries::rawZYXSparse==null [  ERROR  ]");
            return null;
        }

        if (rawZYXSparse.isEmpty()) {
            FrameGui.LOGGER.log(
                    Level.SEVERE,
                    "ConvContinueSeries::rawZYXSparse is Empty [  ERROR  ]");
            return null;
        }
        //</editor-fold>

        double[] arrayDouble = new double[rawZYXSparse.getSizeOfCoorList()];

        int it = 0;
        int i = 0;
        int iz, iy, ix;
        Map.Entry entryZ, entryY, entryX;
        HashMap hashtableY, hashtableX;
        Iterator iteratorZ, iteratorY, iteratorX;
        double value;

        if (series.seriesUnit.unitY == SeriesUnits.UNIT_SUV_BW) {

            iteratorZ = rawZYXSparse.hashtableZ.entrySet().iterator();
            while (iteratorZ.hasNext()) {
                entryZ = (Map.Entry) iteratorZ.next();
                iz = (Integer) entryZ.getKey();
                hashtableY = (HashMap) entryZ.getValue();

                iteratorY = hashtableY.entrySet().iterator();
                while (iteratorY.hasNext()) {
                    entryY = (Map.Entry) iteratorY.next();
                    iy = (Integer) entryY.getKey();
                    hashtableX = (HashMap) entryY.getValue();

                    iteratorX = hashtableX.entrySet().iterator();
                    while (iteratorX.hasNext()) {
                        entryX = (Map.Entry) iteratorX.next();
                        ix = (Integer) entryX.getKey();

                        value = rawZYXFloat.getTZYX(it, iz, iy, ix);
                        arrayDouble[i++] = PT_NM.convCpsVxToSuvBw(series, value);  // difference here
                    }
                }
            }
            
        } else if (series.seriesUnit.unitY == SeriesUnits.UNIT_SUV_LBM) {

            iteratorZ = rawZYXSparse.hashtableZ.entrySet().iterator();
            while (iteratorZ.hasNext()) {
                entryZ = (Map.Entry) iteratorZ.next();
                iz = (Integer) entryZ.getKey();
                hashtableY = (HashMap) entryZ.getValue();

                iteratorY = hashtableY.entrySet().iterator();
                while (iteratorY.hasNext()) {
                    entryY = (Map.Entry) iteratorY.next();
                    iy = (Integer) entryY.getKey();
                    hashtableX = (HashMap) entryY.getValue();

                    iteratorX = hashtableX.entrySet().iterator();
                    while (iteratorX.hasNext()) {
                        entryX = (Map.Entry) iteratorX.next();
                        ix = (Integer) entryX.getKey();

                        value = rawZYXFloat.getTZYX(it, iz, iy, ix);
                        arrayDouble[i++] = PT_NM.convCpsVxToSuvLbm(series, value);  // difference here
                    }
                }
            }
            
        } else if (series.seriesUnit.unitY == SeriesUnits.UNIT_SUV_BSA) {

            iteratorZ = rawZYXSparse.hashtableZ.entrySet().iterator();
            while (iteratorZ.hasNext()) {
                entryZ = (Map.Entry) iteratorZ.next();
                iz = (Integer) entryZ.getKey();
                hashtableY = (HashMap) entryZ.getValue();

                iteratorY = hashtableY.entrySet().iterator();
                while (iteratorY.hasNext()) {
                    entryY = (Map.Entry) iteratorY.next();
                    iy = (Integer) entryY.getKey();
                    hashtableX = (HashMap) entryY.getValue();

                    iteratorX = hashtableX.entrySet().iterator();
                    while (iteratorX.hasNext()) {
                        entryX = (Map.Entry) iteratorX.next();
                        ix = (Integer) entryX.getKey();

                        value = rawZYXFloat.getTZYX(it, iz, iy, ix);
                        arrayDouble[i++] = PT_NM.convCpsVxToSuvBsa(series, value);  // difference here
                    }
                }
            }
            
        } else if (series.seriesUnit.unitY == SeriesUnits.UNIT_SUV_IBW) {

            iteratorZ = rawZYXSparse.hashtableZ.entrySet().iterator();
            while (iteratorZ.hasNext()) {
                entryZ = (Map.Entry) iteratorZ.next();
                iz = (Integer) entryZ.getKey();
                hashtableY = (HashMap) entryZ.getValue();

                iteratorY = hashtableY.entrySet().iterator();
                while (iteratorY.hasNext()) {
                    entryY = (Map.Entry) iteratorY.next();
                    iy = (Integer) entryY.getKey();
                    hashtableX = (HashMap) entryY.getValue();

                    iteratorX = hashtableX.entrySet().iterator();
                    while (iteratorX.hasNext()) {
                        entryX = (Map.Entry) iteratorX.next();
                        ix = (Integer) entryX.getKey();

                        value = rawZYXFloat.getTZYX(it, iz, iy, ix);
                        arrayDouble[i++] = PT_NM.convCpsVxToSuvIbw(series, value);  // difference here
                    }
                }
            }
            
        } else if (series.seriesUnit.unitY == SeriesUnits.UNIT_kBqmL) {

            iteratorZ = rawZYXSparse.hashtableZ.entrySet().iterator();
            while (iteratorZ.hasNext()) {
                entryZ = (Map.Entry) iteratorZ.next();
                iz = (Integer) entryZ.getKey();
                hashtableY = (HashMap) entryZ.getValue();

                iteratorY = hashtableY.entrySet().iterator();
                while (iteratorY.hasNext()) {
                    entryY = (Map.Entry) iteratorY.next();
                    iy = (Integer) entryY.getKey();
                    hashtableX = (HashMap) entryY.getValue();

                    iteratorX = hashtableX.entrySet().iterator();
                    while (iteratorX.hasNext()) {
                        entryX = (Map.Entry) iteratorX.next();
                        ix = (Integer) entryX.getKey();

                        value = rawZYXFloat.getTZYX(it, iz, iy, ix);
                        arrayDouble[i++] = PT_NM.convCpsVxToBqmL(series, value) / 1_000d;  // difference here
                    }
                }
            }
            
        } else {

            iteratorZ = rawZYXSparse.hashtableZ.entrySet().iterator();
            while (iteratorZ.hasNext()) {
                entryZ = (Map.Entry) iteratorZ.next();
                iz = (Integer) entryZ.getKey();
                hashtableY = (HashMap) entryZ.getValue();

                iteratorY = hashtableY.entrySet().iterator();
                while (iteratorY.hasNext()) {
                    entryY = (Map.Entry) iteratorY.next();
                    iy = (Integer) entryY.getKey();
                    hashtableX = (HashMap) entryY.getValue();

                    iteratorX = hashtableX.entrySet().iterator();
                    while (iteratorX.hasNext()) {
                        entryX = (Map.Entry) iteratorX.next();
                        ix = (Integer) entryX.getKey();

                        arrayDouble[i++] = rawZYXFloat.getTZYX(0, iz, iy, ix);
                    }
                }
            }
        } 

        return arrayDouble;
    }
}
