/*
 * USE OF THE LIFEx SOFTWARE IS SUBJECT TO THE FOLLOWING TERMS AND CONDITIONS. 
 * IF YOU DO NOT ACCEPT THESE TERMS AND CONDITIONS, DO NOT DOWNLOAD THE SOFTWARE. 
 * NON-PROFESSIONAL USERS ARE NOTIFIED THAT THE SOFTWARE AND THE RISKS INHERENT TO ITS USE,
 * MODIFICATION AND/OR DEVELOPMENT AND REPRODUCTION ARE NOT COVERED BY ANY WARRANTY ON THE PART OF THE CEA. 
 * IT IS HIGHLY RECOMMENDED THAT YOU ONLY LOAD AND/OR USE THE SOFTWARE IF YOU HAVE SPECIALIZED IN-DEPTH INFORMATION TECHNOLOGY KNOW-HOW.
 * THE COMMISSARIAT A L’ENERGIE ATOMIQUE ET AUX ENERGIES ALTERNATIVES (FRENCH ATOMIC ENERGY AGENCY), 
 * a public scientific, technical and industrial establishment, 
 * having its registered office at 25 rue Leblanc – bâtiment le Ponant D – 75015 Paris cedex, 
 * hereinafter referred to as "CEA", hereby licenses the right to use the LIFEx software as per the terms and conditions set out below.  
 */
package bardp.protocol.protocolTexture.traditional;

import bardp.gui.FrameGui;
import bardp.img.coor.CoorList;
import bardp.img.coor.CoorRange;
import bardp.raw.RawSparseZYX;
import bardp.raw.RawTZYX;
import bardp.roi.calc.RoiDilateErode;
import bardp.roi.calc.RoiErodeProcess;
import bardp.roi.calc.RoiKernel;
import bardp.session.patient.study.series.SeriesModel;
import bardp.session.patient.study.series.SeriesUnits;
import bardp.session.patient.study.series.load.loadDcm.PT_NM.PT_NM;
import java.util.ArrayList;
import java.util.logging.Level;

 /**
 * <p>
 * Title: Local Image Feature Extraction (LIFEx)</p>
 * <p>
 * Description: erode of spherical 1 voxel until zero size and give statistic of all of
 * result rings</p>
 * <p>
 * Copyright: Copyright (c) 2016</p>
 * <p>
 * Company: Inserm/CNRS/CEA</p>
 * <p>
 * @author Christophe Nioche, Inserm at lifexsoft.org
 * @version 6.0
 */
public class RimController implements Runnable {

    public RimController(
            final RawTZYX rawZYX,
            final RawSparseZYX roiZYX,
            final short roiIndex
    ) {
        this.rawZYX = rawZYX;
        this.roiZYX = roiZYX;
        this.roiIndex = roiIndex;
    }

    @Override
    public void run() {
        long start_time = System.currentTimeMillis();
        FrameGui.LOGGER.log(Level.INFO, "Rim processing ... [ start ]");
        final int it = 0;
        rimResults.clear();

        float kDim = 1; // diameter (unit millimeter) of kernel of sphere

        RawSparseZYX roiZYXCopy0 = roiZYX.createCopy();
        RawTZYX roiZYXCopy1;
        RawTZYX roiZYXCopyRing;
        CoorRange rangeBounds = roiZYXCopy0.getBounds(roiIndex);
        int roiSize = roiZYXCopy0.getSizeOfCoorList();

        //<editor-fold defaultstate="collapsed" desc="kDim and kernel management">
        if (rangeBounds.max.getZ() - rangeBounds.min.getZ() == 0) {
            this.kDimZ = 1;
        } else {
            this.kDimZ = (short) (Math.round(kDim / roiZYX.iopIppSp.spModel.getZSpacing()));
        }

        this.kDimY = (short) (Math.round(kDim / roiZYX.iopIppSp.spModel.getYSpacing()));
        this.kDimX = (short) (Math.round(kDim / roiZYX.iopIppSp.spModel.getXSpacing()));

        if (kDimZ > 1) {
            if ((kDimZ % 2) == 0) {
                kDimZ++;
            }
            if (kDimZ < 3) {
                kDimZ = 3;
            }
        } else {
          kDimZ = 1;
        }

        if ((kDimY % 2) == 0) {
            kDimY++;
        }
        if (kDimY < 3) {
            kDimY = 3;
        }

        if ((kDimX % 2) == 0) {
            kDimX++;
        }
        if (kDimX < 3) {
            kDimX = 3;
        }

        // create kernel
        kernel = (new RoiKernel()).run(kDimX, kDimY, kDimZ);
        //</editor-fold>

        // RoiMain roiMainResult1; // debug: for visual layer roiMain
        // complete ROI
        ConventionalRimExtractRawZYX extractRawZYX = new ConventionalRimExtractRawZYX(rawZYX, roiZYXCopy0.getCoorList());
        rimResults.add(extractRawZYX.run());

        while (roiSize > 0) {
            int[] roiZYXinDenseFormat0 = roiZYXCopy0.convZYXInDenseFormatShort();
            CoorList coorList0 = roiZYXCopy0.getCoorList();

            // to delete
            /* debug: for visual layer roiMain
            roiMainResult1 = new PicRoiCreate().createROIManual("erod" + roiSize);
            roiMainResult1.setColor(Color.MAGENTA);
            roiMainResult1.setAlpha2D(128);
            roiMainResult1.roiModel.roiZYX = new RawSparse(
                    rawZYX.nz, rawZYX.ny, rawZYX.nx,
                    rawZYX.iopIppSp.spModel.spacing,
                    rawZYX.iopIppSp.ippModel.ipp
            );*/
            roiZYXCopy1 = new RawTZYX(
                    1, rawZYX.nz, rawZYX.ny, rawZYX.nx,
                    rawZYX.iopIppSp.spModel.spacing.z,
                    rawZYX.iopIppSp.spModel.spacing.y,
                    rawZYX.iopIppSp.spModel.spacing.x
            );
            (new RoiErodeProcess(
                    rangeBounds, kernel,
                    kDimZ, kDimY, kDimX,
                    roiZYXinDenseFormat0, roiIndex,
                    // roiMainResult1.roiModel.roiIndex, roiMainResult1.roiModel.roiZYX, // debug: for viisual layer roiMain
                    roiIndex, roiZYXCopy1,
                    RoiDilateErode.MESSAGE_HIDE
            )).run();

            //roiMainResult1.updateStatistics_Image2D_Image3D_Mip_FromAllSeries(); // debug: for visual layer picRoMain
            // ring pocessing
            CoorList coorList1 = roiZYXCopy1.getCoorList(roiIndex);
            CoorList coorListRing = coorList0.getListWithoutOtherList(coorList1);

            /* // debug: for visual layer roiMain
            roiMain roiMainRing = new RoiCreate().createROIManual("ring" + roiSize);
            roiMainRing.setColor(Color.MAGENTA);
            roiMainRing.setAlpha2D(128);
            roiMainRing.roiModel.roiZYX = new RawSparse(
                    rawZYX.nz, rawZYX.ny, rawZYX.nx,
                    rawZYX.iopIppSp.spModel.spacing,
                    rawZYX.iopIppSp.ippModel.ipp
            );*/
            roiZYXCopyRing = new RawTZYX(
                    1, rawZYX.nz, rawZYX.ny, rawZYX.nx,
                    rawZYX.iopIppSp.spModel.spacing.z,
                    rawZYX.iopIppSp.spModel.spacing.y,
                    rawZYX.iopIppSp.spModel.spacing.x
            );
            roiZYXCopyRing.set(it, coorListRing, roiIndex);
            // roiMainRing.updateStatistics_Image2D_Image3D_Mip_FromAllSeries();// debug: for visual layer roiMain

            extractRawZYX = new ConventionalRimExtractRawZYX(rawZYX, coorListRing);
            rimResults.add(extractRawZYX.run());

            roiSize = coorList1.size();
            roiZYXCopy0 = roiZYXCopy1.getRawSparse(it, roiIndex);
        }

        FrameGui.LOGGER.log(Level.INFO, "Rim processing ... [ end ]");
        long end_time = System.currentTimeMillis();
        long difference = end_time - start_time;
        FrameGui.LOGGER.log(Level.INFO, "Time calculation of Rim proceding: {0} sec", (difference / 1_000d));
    }

    @Override
    public String toString() {
        String str = "";
        for (ConventionalRimModel rimResult : rimResults) {
            str += "CONV_RIMmin:" + rimResult.min + "\n";
            str += "CONV_RIMmax:" + rimResult.max + "\n";
            str += "CONV_RIMn:" + rimResult.n + "\n";
            str += "CONV_RIMcm3:" + rimResult.cm3 + "\n";
            str += "CONV_RIMmean:" + rimResult.mean + "\n";
            str += "CONV_RIMstdev:" + rimResult.stdev + "\n";
            str += "CONV_RIMsum:" + rimResult.sum + "\n";
            str += "|";
        }
        return str;
    }

    /**
     * get Rim results of all rings following by 1 voxel erosion
     *
     * @return
     */
    public ArrayList<ConventionalRimModel> getRimResults() {
        return rimResults;
    }

    private final RawTZYX rawZYX;
    private final RawSparseZYX roiZYX;
    private final short roiIndex;

    private boolean[][][] kernel;

    /**
     * kernel size of operator (in mm)
     */
    private short kDimX;
    private short kDimY;
    private short kDimZ;

    /**
     * rim Results (see RimModel for structure)
     */
    private final ArrayList<ConventionalRimModel> rimResults = new ArrayList<>();
}
